from BTrees.OOBTree import OOBTree
from gzip import GzipFile
from io import BytesIO
from plone.memoize import ram
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from plone.base.interfaces import IPloneSiteRoot
from plone.base.interfaces import ISiteSchema
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import getUtility
from zope.publisher.interfaces import NotFound


def _render_cachekey(fun, self):
    # Cache by filename
    mtool = getToolByName(self.context, "portal_membership")
    if not mtool.isAnonymousUser():
        raise ram.DontCache

    url = self.context.absolute_url()
    catalog = getToolByName(self.context, "portal_catalog")
    counter = catalog.getCounter()
    return f"{url}/{self.filename}/{counter}"


class SiteMapView(BrowserView):
    """Creates the sitemap as explained in the specifications.

    http://www.sitemaps.org/protocol.php
    """

    template = ViewPageTemplateFile("sitemap.xml")

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.filename = "sitemap.xml.gz"

    def objects(self):
        """Returns the data to create the sitemap."""
        catalog = getToolByName(self.context, "portal_catalog")
        query = {}
        utils = getToolByName(self.context, "plone_utils")
        query["portal_type"] = utils.getUserFriendlyTypes()
        registry = getUtility(IRegistry)
        typesUseViewActionInListings = frozenset(
            registry.get("plone.types_use_view_action_in_listings", [])
        )

        is_plone_site_root = IPloneSiteRoot.providedBy(self.context)
        if not is_plone_site_root:
            query["path"] = "/".join(self.context.getPhysicalPath())

        query["is_default_page"] = True
        default_page_modified = OOBTree()
        for item in catalog.searchResults(query):
            key = item.getURL().rsplit("/", 1)[0]
            value = (item.modified.micros(), item.modified.ISO8601())
            default_page_modified[key] = value

        # The plone site root is not catalogued.
        if is_plone_site_root:
            loc = self.context.absolute_url()
            date = self.context.modified()
            # Comparison must be on GMT value
            modified = (date.micros(), date.ISO8601())
            default_modified = default_page_modified.get(loc, None)
            if default_modified is not None:
                modified = max(modified, default_modified)
            lastmod = modified[1]
            yield {
                "loc": loc,
                "lastmod": lastmod,
                # 'changefreq': 'always',
                #  hourly/daily/weekly/monthly/yearly/never
                # 'prioriy': 0.5, # 0.0 to 1.0
            }

        query["is_default_page"] = False
        for item in catalog.searchResults(query):
            loc = item.getURL()
            date = item.modified
            # Comparison must be on GMT value
            modified = (date.micros(), date.ISO8601())
            default_modified = default_page_modified.get(loc, None)
            if default_modified is not None:
                modified = max(modified, default_modified)
            lastmod = modified[1]
            if item.portal_type in typesUseViewActionInListings:
                loc += "/view"
            yield {
                "loc": loc,
                "lastmod": lastmod,
                # 'changefreq': 'always',
                #  hourly/daily/weekly/monthly/yearly/never
                # 'prioriy': 0.5, # 0.0 to 1.0
            }

    @ram.cache(_render_cachekey)
    def generate(self):
        """Generates the Gzipped sitemap."""
        xml = self.template()
        fp = BytesIO()
        gzip = GzipFile(self.filename, "wb", 9, fp)
        if isinstance(xml, str):
            xml = xml.encode("utf8")
        gzip.write(xml)
        gzip.close()
        data = fp.getvalue()
        fp.close()
        return data

    def __call__(self):
        """Checks if the sitemap feature is enable and returns it."""
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSchema, prefix="plone")
        if not settings.enable_sitemap:
            raise NotFound(self.context, self.filename, self.request)

        self.request.response.setHeader("Content-Type", "application/octet-stream")
        return self.generate()



from plone.app.layout.navigation.root import getNavigationRootObject
from plone.app.layout.testing import INTEGRATION_TESTING

import unittest


class NavigationRootTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def test_getNavigationRootObject_no_context(self):
        """
        If you don't know the context then you also don't know what the
        navigation root is.
        """
        self.portal = self.layer["portal"]
        self.assertEqual(None, getNavigationRootObject(None, self.portal))



from DateTime import DateTime
from gzip import GzipFile
from io import BytesIO
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.app.layout.testing import INTEGRATION_TESTING
from plone.app.testing import login
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.base.utils import safe_text
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from plone.base.interfaces import ISearchSchema
from plone.base.interfaces import ISiteSchema
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.interface import alsoProvides
from zope.publisher.interfaces import INotFound

import unittest


class SiteMapTestCase(unittest.TestCase):
    """base test case with convenience methods for all sitemap tests"""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        registry = getUtility(IRegistry)
        self.site_settings = registry.forInterface(ISiteSchema, prefix="plone")
        self.site_settings.enable_sitemap = True
        self.sitemap = getMultiAdapter(
            (self.portal, self.portal.REQUEST), name="sitemap.xml.gz"
        )
        self.wftool = getToolByName(self.portal, "portal_workflow")

        # we need to explizitly set a workflow cause we can't rely on the
        # test environment.
        # `instance test -m plone.app.layout`:
        # wftool._default_chain == 'simple_publication_workflow'
        # `instance test -m plone.app`:
        # wftool._default_chain == 'plone_workflow'
        self.wftool.setChainForPortalTypes(["Document"], "simple_publication_workflow")

        self.site_properties = getToolByName(
            self.portal, "portal_properties"
        ).site_properties

        # setup private content that isn't accessible for anonymous
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.invokeFactory(id="private", type_name="Document")
        private = self.portal.private
        self.assertTrue("private" == self.wftool.getInfoFor(private, "review_state"))

        # setup published content that is accessible for anonymous
        self.portal.invokeFactory(id="published", type_name="Document")
        published = self.portal.published
        self.wftool.doActionFor(published, "publish")
        self.assertTrue(
            "published" == self.wftool.getInfoFor(published, "review_state")
        )

        # setup pending content that isn't accessible for anonymous
        self.portal.invokeFactory(id="pending", type_name="Document")
        pending = self.portal.pending
        self.wftool.doActionFor(pending, "submit")
        self.assertTrue("pending" == self.wftool.getInfoFor(pending, "review_state"))
        logout()

    def uncompress(self, sitemapdata):
        sio = BytesIO(sitemapdata)
        unziped = GzipFile(fileobj=sio)
        xml = unziped.read()
        unziped.close()
        return safe_text(xml)

    def test_disabled(self):
        """
        If the sitemap is disabled throws a 404 error.
        """
        self.site_settings.enable_sitemap = False
        try:
            self.sitemap()
        except Exception as e:
            # zope2 and repoze.zope2 use different publishers and raise
            # different exceptions. but both implement INotFound.
            self.assertTrue(INotFound.providedBy(e))
        else:
            self.fail("The disabled sitemap view has to raise NotFound!")

    def test_authenticated_before_anonymous(self):
        """
        Requests for the sitemap by authenticated users are not cached.
        anomymous users get a uncached sitemap that only contains content
        that they are supposed to see.
        """

        # first round as an authenticated (manager)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)
        xml = self.uncompress(self.sitemap())
        self.assertTrue("<loc>http://nohost/plone/private</loc>" in xml)
        self.assertTrue("<loc>http://nohost/plone/pending</loc>" in xml)
        self.assertTrue("<loc>http://nohost/plone/published</loc>" in xml)

        # second round as anonymous
        logout()
        xml = self.uncompress(self.sitemap())
        self.assertFalse("<loc>http://nohost/plone/private</loc>" in xml)
        self.assertFalse("<loc>http://nohost/plone/pending</loc>" in xml)
        self.assertTrue("<loc>http://nohost/plone/published</loc>" in xml)

    def test_anonymous_before_authenticated(self):
        """
        Requests for the sitemap by anonymous users are cached.
        authenticated users get a uncached sitemap. Test that the cached
        Sitemap is not delivered to authenticated users.
        """

        # first round as anonymous
        xml = self.uncompress(self.sitemap())
        self.assertFalse("<loc>http://nohost/plone/private</loc>" in xml)
        self.assertFalse("<loc>http://nohost/plone/pending</loc>" in xml)
        self.assertTrue("<loc>http://nohost/plone/published</loc>" in xml)

        # second round as an authenticated (manager)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)
        xml = self.uncompress(self.sitemap())
        self.assertTrue("<loc>http://nohost/plone/private</loc>" in xml)
        self.assertTrue("<loc>http://nohost/plone/pending</loc>" in xml)
        self.assertTrue("<loc>http://nohost/plone/published</loc>" in xml)

    def test_changed_catalog(self):
        """
        The sitemap is generated from the catalog. If the catalog changes, a
        new sitemap has to be generated.
        """

        xml = self.uncompress(self.sitemap())
        self.assertFalse("<loc>http://nohost/plone/pending</loc>" in xml)

        # changing the workflow state
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        pending = self.portal.pending
        self.wftool.doActionFor(pending, "publish")
        logout()

        xml = self.uncompress(self.sitemap())
        self.assertTrue("<loc>http://nohost/plone/pending</loc>" in xml)

        # removing content
        login(self.portal, TEST_USER_NAME)
        self.portal.manage_delObjects(
            [
                "published",
            ]
        )
        logout()

        xml = self.uncompress(self.sitemap())
        self.assertFalse("<loc>http://nohost/plone/published</loc>" in xml)

    def test_navroot(self):
        """
        Sitemap generated from an INavigationRoot
        """
        # setup navroot content that is accessible for anonymous
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)
        self.portal.invokeFactory(id="navroot", type_name="Folder")
        navroot = self.portal.navroot
        self.wftool.doActionFor(navroot, "publish")
        self.assertTrue("published" == self.wftool.getInfoFor(navroot, "review_state"))
        alsoProvides(navroot, INavigationRoot)
        navroot.invokeFactory(id="published", type_name="Document")
        published = navroot.published
        self.wftool.doActionFor(published, "publish")
        self.assertTrue(
            "published" == self.wftool.getInfoFor(published, "review_state")
        )
        logout()

        sitemap = getMultiAdapter(
            (self.portal.navroot, self.portal.REQUEST), name="sitemap.xml.gz"
        )
        xml = self.uncompress(sitemap())
        self.assertFalse("<loc>http://nohost/plone/published</loc>" in xml)
        self.assertTrue("<loc>http://nohost/plone/navroot</loc>" in xml)
        self.assertTrue("<loc>http://nohost/plone/navroot/published</loc>" in xml)

    def test_types_not_searched(self):
        """
        Test that types_not_searched is respected
        """
        # Set News Items not to be searchable (more likely Images)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)
        self.portal.invokeFactory(id="newsitem", type_name="News Item")
        newsitem = self.portal.newsitem
        self.wftool.doActionFor(newsitem, "publish")
        self.assertTrue("published" == self.wftool.getInfoFor(newsitem, "review_state"))
        registry = getUtility(IRegistry)
        search_settings = registry.forInterface(ISearchSchema, prefix="plone")
        search_settings.types_not_searched = ("News Item",)
        logout()

        xml = self.uncompress(self.sitemap())
        self.assertFalse("<loc>http://nohost/plone/newsitem</loc>" in xml)

    def test_typesUseViewActionInListings(self):
        """
        Test that typesUseViewActionInListings is respected
        """
        # Set News Items not to be searchable (more likely Images)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)
        self.portal.invokeFactory(id="newsitem", type_name="News Item")
        newsitem = self.portal.newsitem
        self.wftool.doActionFor(newsitem, "publish")
        self.assertTrue("published" == self.wftool.getInfoFor(newsitem, "review_state"))
        registry = getUtility(IRegistry)
        registry["plone.types_use_view_action_in_listings"] = ["News Item"]

        logout()

        xml = self.uncompress(self.sitemap())
        self.assertTrue("<loc>http://nohost/plone/newsitem/view</loc>" in xml)

    def test_default_pages(self):
        """
        Default pages should show up at their parent's url with the greater of
        their or their parent's modification time.
        """

        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)
        self.portal.invokeFactory(id="folder", type_name="Folder")
        folder = self.portal.folder
        folder.default_page = "default"
        self.wftool.doActionFor(folder, "publish")
        self.assertTrue("published" == self.wftool.getInfoFor(folder, "review_state"))

        folder.invokeFactory(id="default", type_name="Document")
        default = folder.default
        self.wftool.doActionFor(default, "publish")
        self.assertTrue("published" == self.wftool.getInfoFor(default, "review_state"))
        self.assertTrue(self.portal.plone_utils.isDefaultPage(default))
        default.modification_date = DateTime("2001-01-01")
        folder.modification_date = DateTime("2000-01-01")
        self.portal.portal_catalog.reindexObject(
            folder, idxs=["modified", "is_default_page", "effective"]
        )
        self.portal.portal_catalog.reindexObject(
            default, idxs=["modified", "is_default_page", "effective"]
        )
        self.portal.default_page = "published"
        self.portal.portal_catalog.reindexObject(
            self.portal.published, idxs=["modified", "is_default_page", "effective"]
        )
        logout()

        xml = self.uncompress(self.sitemap())
        self.assertFalse("<loc>http://nohost/plone/folder/default</loc>" in xml)
        self.assertTrue("<loc>http://nohost/plone/folder</loc>" in xml)
        self.assertTrue("<lastmod>2001-01-01T" in xml)
        self.assertTrue("<loc>http://nohost/plone</loc>" in xml)
        self.assertFalse("<loc>http://nohost/plone/published</loc>" in xml)



from plone.app.layout.testing import INTEGRATION_TESTING
from Products.CMFCore.utils import getToolByName

import unittest


class TestToolsView(unittest.TestCase):
    """Tests the global tools view."""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.folder = self.portal["Members"]
        self.view = self.folder.restrictedTraverse("@@plone_tools")

    def test_actions(self):
        self.assertEqual(
            self.view.actions(), getToolByName(self.folder, "portal_actions")
        )

    def test_catalog(self):
        self.assertEqual(
            self.view.catalog(), getToolByName(self.folder, "portal_catalog")
        )

    def test_membership(self):
        self.assertEqual(
            self.view.membership(), getToolByName(self.folder, "portal_membership")
        )

    def test_properties(self):
        self.assertEqual(
            self.view.properties(), getToolByName(self.folder, "portal_properties")
        )

    def test_types(self):
        self.assertEqual(self.view.types(), getToolByName(self.folder, "portal_types"))

    def test_url(self):
        self.assertEqual(self.view.url(), getToolByName(self.folder, "portal_url"))

    def test_workflow(self):
        self.assertEqual(
            self.view.workflow(), getToolByName(self.folder, "portal_workflow")
        )



from AccessControl import getSecurityManager
from Acquisition import aq_inner
from DateTime import DateTime
from plone.app.content.browser.interfaces import IFolderContentsView
from plone.app.layout.globals.interfaces import IViewView
from plone.app.layout.viewlets import ViewletBase
from plone.app.multilingual.browser.vocabularies import translated_languages
from plone.app.multilingual.interfaces import ITranslatable
from plone.app.multilingual.interfaces import ITranslationManager
from plone.base.interfaces import ISecuritySchema
from plone.base.interfaces import ISiteSchema
from plone.base.utils import base_hasattr
from plone.memoize.instance import memoize
from plone.memoize.view import memoize_contextless
from plone.protect.authenticator import createToken
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import _checkPermission
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.WorkflowCore import WorkflowException
from Products.CMFEditions.Permissions import AccessPreviousVersions
from Products.CMFPlone import PloneMessageFactory as _
from Products.CMFPlone.utils import log
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from urllib.parse import urlencode
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.deprecation import deprecation

import logging
import pkg_resources


try:
    pkg_resources.get_distribution("plone.app.relationfield")
except pkg_resources.DistributionNotFound:
    HAS_RELATIONFIELD = False
else:
    from plone.app.relationfield.behavior import IRelatedItems

    HAS_RELATIONFIELD = True

# XXX needs refactoring, since Plone 5 we have PAM in core.
HAS_PAM = True


class DocumentActionsViewlet(ViewletBase):

    index = ViewPageTemplateFile("document_actions.pt")

    def update(self):
        super().update()

        self.context_state = getMultiAdapter(
            (self.context, self.request), name="plone_context_state"
        )
        self.actions = self.context_state.actions("document_actions")


class DocumentBylineViewlet(ViewletBase):

    index = ViewPageTemplateFile("document_byline.pt")

    def update(self):
        super().update()
        self.anonymous = self.portal_state.anonymous()

    @property
    @deprecation.deprecate("The context_state property is unused and will be removed in Plone 7")
    def context_state(self):
        return getMultiAdapter((self.context, self.request), name="plone_context_state")

    @property
    @deprecation.deprecate("The has_pam property is unused and will be removed in Plone 7")
    def has_pam(self):
        return HAS_PAM

    @property
    @memoize_contextless
    def portal_membership(self):
        return getToolByName(self.context, "portal_membership")

    def show(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(
            ISiteSchema,
            prefix="plone",
        )
        return not self.anonymous or settings.display_publication_date_in_byline

    def show_about(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(
            ISecuritySchema,
            prefix="plone",
        )
        return not self.anonymous or settings.allow_anon_views_about

    @deprecation.deprecate("The creator method is unused and will be removed in Plone 7")
    def creator(self):
        return self.context.Creator()

    @deprecation.deprecate("The author method is unused and will be removed in Plone 7")
    def author(self):
        membership = getToolByName(self.context, "portal_membership")
        return membership.getMemberInfo(self.creator())

    @deprecation.deprecate("The authorname method is unused and will be removed in Plone 7")
    def authorname(self):
        author = self.author()
        return author and author["fullname"] or self.creator()

    @memoize
    def get_member_info(self, user_id):
        return self.portal_membership.getMemberInfo(user_id)

    def get_url_path(self, user_id):
        if self.get_member_info(user_id) is None:
            return ""
        if "/" in user_id:
            qs = urlencode({"author": user_id})
            return f"author/?{qs}"
        return f"author/{user_id}"

    def get_fullname(self, user_id):
        info = self.get_member_info(user_id)
        if info is None:
            return user_id
        return info.get("fullname") or user_id

    def show_modification_date(self):
        return not self.context.effective_date or (
            self.context.effective_date.Date() < self.context.modification_date.Date()
        )

    def isExpired(self):
        if base_hasattr(self.context, "expires"):
            return self.context.expires().isPast()
        return False

    @deprecation.deprecate("The toLocalizedTime method is unused and will be removed in Plone 7")
    def toLocalizedTime(self, time, long_format=None, time_only=None):
        """Convert time to localized time"""
        util = getToolByName(self.context, "translation_service")
        return util.ulocalized_time(
            time, long_format, time_only, self.context, domain="plonelocales"
        )

    def pub_date(self):
        """Return object effective date.

        Return None if publication date is switched off in global site settings
        or if Effective Date is not set on object.
        """
        # check if we are allowed to display publication date
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSchema, prefix="plone")

        if not settings.display_publication_date_in_byline:
            return None

        # check if we have Effective Date set
        date = self.context.EffectiveDate()
        if not date or date == "None":
            return None

        return DateTime(date)

    @deprecation.deprecate("The get_translations method is unused and will be removed in Plone 7")
    def get_translations(self):
        cts = []
        if ITranslatable.providedBy(self.context):
            t_langs = translated_languages(self.context)
            context_translations = ITranslationManager(self.context).get_translations()
            for lang in t_langs:
                cts.append(
                    dict(
                        lang_native=lang.title,
                        url=context_translations[lang.value].absolute_url(),
                    )
                )
        return cts


class HistoryByLineView(BrowserView):
    """DocumentByLine information for content history view"""

    index = ViewPageTemplateFile("history_view.pt")

    def update(self):
        context = self.context
        self.portal_state = getMultiAdapter(
            (context, self.request), name="plone_portal_state"
        )
        self.context_state = getMultiAdapter(
            (self.context, self.request), name="plone_context_state"
        )
        self.anonymous = self.portal_state.anonymous()
        self.has_pam = HAS_PAM

    def __call__(self):
        self.update()

        return self.index()

    def show(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(
            ISecuritySchema,
            prefix="plone",
        )
        return not self.anonymous or settings.allow_anon_views_about

    def show_history(self):
        has_access_preview_versions_permission = _checkPermission(
            "CMFEditions: Access previous versions", self.context
        )
        if not has_access_preview_versions_permission:
            return False
        if IViewView.providedBy(self.__parent__):
            return True
        if IFolderContentsView.providedBy(self.__parent__):
            return True
        return False

    def locked_icon(self):
        if not getSecurityManager().checkPermission(
            "Modify portal content", self.context
        ):
            return ""

        locked = False
        lock_info = queryMultiAdapter(
            (self.context, self.request), name="plone_lock_info"
        )
        if lock_info is not None:
            locked = lock_info.is_locked()
        else:
            context = aq_inner(self.context)
            lockable = getattr(context.aq_explicit, "wl_isLocked", None) is not None
            locked = lockable and context.wl_isLocked()

        if not locked:
            return ""

        portal = self.portal_state.portal()
        icon = portal.restrictedTraverse("lock_icon.png")
        return icon.tag(title="Locked")

    def creator(self):
        return self.context.Creator()

    def author(self):
        membership = getToolByName(self.context, "portal_membership")
        return membership.getMemberInfo(self.creator())

    def authorname(self):
        author = self.author()
        return author and author["fullname"] or self.creator()

    def isExpired(self):
        if base_hasattr(self.context, "expires"):
            return self.context.expires().isPast()
        return False

    def toLocalizedTime(self, time, long_format=None, time_only=None):
        """Convert time to localized time"""
        util = getToolByName(self.context, "translation_service")
        return util.ulocalized_time(
            time, long_format, time_only, self.context, domain="plonelocales"
        )

    def pub_date(self):
        """Return object effective date.

        Return None if publication date is switched off in global site settings
        or if Effective Date is not set on object.
        """
        # check if we are allowed to display publication date
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSchema, prefix="plone")

        if not settings.display_publication_date_in_byline:
            return None

        # check if we have Effective Date set
        date = self.context.EffectiveDate()
        if not date or date == "None":
            return None

        return DateTime(date)

    def get_translations(self):
        cts = []
        if ITranslatable.providedBy(self.context):
            t_langs = translated_languages(self.context)
            context_translations = ITranslationManager(self.context).get_translations()
            for lang in t_langs:
                cts.append(
                    dict(
                        lang_native=lang.title,
                        url=context_translations[lang.value].absolute_url(),
                    )
                )

        return cts


class ContentRelatedItems(ViewletBase):

    index = ViewPageTemplateFile("document_relateditems.pt")

    def related_items(self):
        context = aq_inner(self.context)
        res = ()

        # Archetypes
        if base_hasattr(context, "getRawRelatedItems"):
            catalog = getToolByName(context, "portal_catalog")
            related = context.getRawRelatedItems()
            if not related:
                return ()
            brains = catalog(UID=related)
            if brains:
                # build a position dict by iterating over the items once
                positions = {v: i for (i, v) in enumerate(related)}
                # We need to keep the ordering intact
                res = list(brains)

                def _key(brain):
                    return positions.get(brain.UID, -1)

                res.sort(key=_key)

        # Dexterity
        if HAS_RELATIONFIELD and IRelatedItems.providedBy(context):
            related = context.relatedItems
            if not related:
                return ()
            res = self.related2brains(related)

        return res

    def related2brains(self, related):
        """Return a list of brains based on a list of relations. Will filter
        relations if the user has no permission to access the content.

        :param related: related items
        :type related: list of relations
        :return: list of catalog brains
        """
        catalog = getToolByName(self.context, "portal_catalog")
        brains = []
        for r in related:
            path = r.to_path
            if path is None:
                # Item was deleted.  The related item should have been cleaned
                # up, but apparently this does not happen.
                continue
            # the query will return an empty list if the user
            # has no permission to see the target object
            brains.extend(catalog(path=dict(query=path, depth=0)))
        return brains


class WorkflowHistoryViewlet(ViewletBase):

    index = ViewPageTemplateFile("review_history.pt")

    @memoize
    def getUserInfo(self, userid):
        actor = dict(fullname=userid)
        mt = getToolByName(self.context, "portal_membership")
        info = mt.getMemberInfo(userid)
        if info is None:
            return dict(actor_home="", actor=actor)

        fullname = info.get("fullname", None)
        if fullname:
            actor["fullname"] = fullname

        return dict(actor=actor, actor_home=f"{self.site_url}/author/{userid}")

    def workflowHistory(self, complete=True):
        """Return workflow history of this context.

        Taken from plone_scripts/getWorkflowHistory.py
        """
        context = aq_inner(self.context)
        # check if the current user has the proper permissions
        if not (
            _checkPermission("Request review", context)
            or _checkPermission("Review portal content", context)
        ):
            return []

        workflow = getToolByName(context, "portal_workflow")
        review_history = []

        try:
            # get total history
            review_history = workflow.getInfoFor(context, "review_history")

            if not complete:
                # filter out automatic transitions.
                review_history = [r for r in review_history if r["action"]]
            else:
                review_history = list(review_history)

            portal_type = context.portal_type
            anon = _("label_anonymous_user", default="Anonymous User")

            for r in review_history:
                r["type"] = "workflow"
                r["transition_title"] = workflow.getTitleForTransitionOnType(
                    r["action"], portal_type
                ) or _("Create")
                r["state_title"] = workflow.getTitleForStateOnType(
                    r["review_state"], portal_type
                )
                actorid = r["actor"]
                r["actorid"] = actorid
                if actorid is None:
                    # action performed by an anonymous user
                    r["actor"] = {"username": anon, "fullname": anon}
                    r["actor_home"] = ""
                else:
                    r.update(self.getUserInfo(actorid))
            review_history.reverse()

        except WorkflowException:
            log(
                "plone.app.layout.viewlets.content: "
                "%s has no associated workflow" % context.absolute_url(),
                severity=logging.DEBUG,
            )

        return review_history


class ContentHistoryViewlet(WorkflowHistoryViewlet):

    index = ViewPageTemplateFile("content_history.pt")

    def revisionHistory(self):
        context = aq_inner(self.context)
        if not _checkPermission(AccessPreviousVersions, context):
            return []

        rt = getToolByName(context, "portal_repository", None)
        if rt is None or not rt.isVersionable(context):
            return []

        context_url = context.absolute_url()
        history = rt.getHistoryMetadata(context)
        portal_diff = getToolByName(context, "portal_diff", None)
        can_diff = (
            portal_diff is not None
            and len(portal_diff.getDiffForPortalType(context.portal_type)) > 0
        )
        can_revert = _checkPermission(
            "CMFEditions: Revert to previous versions", context
        )

        def morphVersionDataToHistoryFormat(vdata, version_id):
            meta = vdata["metadata"]["sys_metadata"]
            userid = meta["principal"]
            token = createToken()
            preview_url = (
                "%s/versions_history_form?version_id=%s&_authenticator=%s#version_preview"
                % (context_url, version_id, token)  # noqa
            )
            info = dict(
                type="versioning",
                action=_("Edited"),
                transition_title=_("Edited"),
                actorid=userid,
                time=meta["timestamp"],
                comments=meta["comment"],
                version_id=version_id,
                preview_url=preview_url,
            )
            if can_diff:
                if version_id > 0:
                    info[
                        "diff_previous_url"
                    ] = "{}/@@history?one={}&two={}&_authenticator={}".format(
                        context_url,
                        version_id,
                        version_id - 1,
                        token,
                    )
                if not rt.isUpToDate(context, version_id):
                    info[
                        "diff_current_url"
                    ] = "{}/@@history?one=current&two={}&_authenticator={}".format(
                        context_url,
                        version_id,
                        token,
                    )
            if can_revert:
                info["revert_url"] = "%s/revertversion" % context_url
            else:
                info["revert_url"] = None
            info.update(self.getUserInfo(userid))
            return info

        # History may be an empty list
        if not history:
            return history

        version_history = []
        retrieve = history.retrieve
        getId = history.getVersionId
        # Count backwards from most recent to least recent
        for i in range(history.getLength(countPurged=False) - 1, -1, -1):
            version_history.append(
                morphVersionDataToHistoryFormat(
                    retrieve(i, countPurged=False), getId(i, countPurged=False)
                )
            )

        return version_history

    def fullHistory(self):
        history = self.workflowHistory() + self.revisionHistory()
        if len(history) == 0:
            return None
        history.sort(key=lambda x: x["time"], reverse=True)
        return history

    def toLocalizedTime(self, time, long_format=None, time_only=None):
        """Convert time to localized time"""
        util = getToolByName(self.context, "translation_service")
        return util.ulocalized_time(
            time, long_format, time_only, self.context, domain="plonelocales"
        )


class ContentHistoryView(ContentHistoryViewlet):
    def __init__(self, context, request):
        super().__init__(context, request, None, None)
        self.update()

    def __call__(self):
        return self.index()



from Acquisition import aq_inner
from plone.app.layout.viewlets import ViewletBase
from plone.app.uuid.utils import uuidToObject
from plone.base.interfaces import ISiteSyndicationSettings
from plone.base.interfaces.syndication import IFeedSettings
from plone.base.utils import safe_bytes
from plone.formwidget.namedfile.converter import b64decode_file
from plone.memoize import ram
from plone.memoize import view
from plone.memoize.compress import xhtml_compress
from plone.registry.interfaces import IRegistry
from plone.base.interfaces import ISecuritySchema
from plone.base.interfaces import ISiteSchema
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from typing import NoReturn
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.schema.interfaces import IVocabularyFactory


def get_language(context, request):
    portal_state = getMultiAdapter((context, request), name="plone_portal_state")
    return portal_state.language()


def render_cachekey(fun, self):
    # Include the name of the viewlet as the underlying cache key only
    # takes the module and function name into account, but not the class
    return "\n".join(
        [
            self.__name__,
            self.site_url,
            get_language(aq_inner(self.context), self.request),
        ]
    )


class FaviconViewlet(ViewletBase):

    _template = ViewPageTemplateFile("favicon.pt")
    mimetype: str
    favicon_path: str

    def init_favicon(self) -> NoReturn:
        registry = getUtility(IRegistry)
        settings: ISiteSchema = registry.forInterface(ISiteSchema, prefix="plone")
        self.mimetype: str = getattr(
            settings, "site_favicon_mimetype", "image/vnd.microsoft.icon"
        )
        cachebust = ""
        if getattr(settings, "site_favicon", False):
            # The user has customized the favicon via the Site configlet.
            filename = b64decode_file(settings.site_favicon)[0]

            cachebust = "?name=" + filename
        # The filename is *always* /favicon.ico, irrespective of the content type,
        # because:
        #
        # 1. Browsers obey the content type over the extension.
        # 2. The actual serving view URL for the favicon is always /favicon.ico,
        #    and this name cannot be overridden from here into the view registered
        #    on CMFPlone, where the *actual* serving of the data takes place.
        # 3. Even if we could somehow override the view, there is no easy way to
        #    register in CMFPlone a different browser view for every icon file
        #    name the user may decide to upload.
        # 4. In many cases client applications just hit /favicon.ico irrespective
        #    of what the HTML says (remember that this specific view is only
        #    responsible for generating the metadata that lets the browser know
        #    where to find the favicon URL).
        #
        # However, to allow for users to change their favicons *and* bust their
        # proxy caches, we do use the favicon filename in the served favicon
        # URL.  This does not cover the case of RSS and podcast apps that access
        # /favicon.ico by custom instead of consulting the HTML, but at least
        # it covers pretty much every browser out there.
        self.favicon_path: str = f"{self.navigation_root_url}/favicon.ico{cachebust}"

    def render(self) -> ViewPageTemplateFile:
        self.init_favicon()
        return xhtml_compress(self._template())


class SearchViewlet(ViewletBase):

    _template = ViewPageTemplateFile("search.pt")

    @ram.cache(render_cachekey)
    def render(self):
        return xhtml_compress(self._template())


class AuthorViewlet(ViewletBase):

    _template = ViewPageTemplateFile("author.pt")

    def update(self):
        super().update()
        self.tools = getMultiAdapter((self.context, self.request), name="plone_tools")

    def show(self):
        anonymous = self.portal_state.anonymous()
        registry = getUtility(IRegistry)
        settings = registry.forInterface(
            ISecuritySchema,
            prefix="plone",
        )
        return not anonymous or settings.allow_anon_views_about

    def render(self):
        if self.show():
            return self._template()
        return ""


class RSSViewlet(ViewletBase):
    def getRssLinks(self, obj):
        settings = IFeedSettings(obj, None)
        if settings is None:
            return []
        factory = getUtility(
            IVocabularyFactory, "plone.app.vocabularies.SyndicationFeedTypes"
        )
        vocabulary = factory(self.context)
        urls = []
        for typ in settings.feed_types:
            try:
                term = vocabulary.getTerm(typ)
            except LookupError:
                continue

            urls.append(
                {
                    "title": f"{obj.Title()} - {safe_bytes(term.title)}",
                    "url": obj.absolute_url() + "/" + term.value,
                }
            )
        return urls

    def update(self):
        super().update()
        self.rsslinks = []
        portal = self.portal_state.portal()
        util = getMultiAdapter((self.context, self.request), name="syndication-util")
        context_state = getMultiAdapter(
            (self.context, self.request), name="plone_context_state"
        )
        if context_state.is_portal_root():
            if util.site_enabled():
                registry = getUtility(IRegistry)
                try:
                    settings = registry.forInterface(ISiteSyndicationSettings)
                except KeyError:
                    return
                if settings.site_rss_items:
                    for uid in settings.site_rss_items:
                        if not uid:
                            continue
                        obj = uuidToObject(uid)
                        if obj is None and uid[0] == "/":
                            obj = portal.restrictedTraverse(uid.lstrip("/"), None)
                        if obj is not None:
                            self.rsslinks.extend(self.getRssLinks(obj))
                self.rsslinks.extend(self.getRssLinks(portal))
        else:
            if util.context_enabled():
                self.rsslinks.extend(self.getRssLinks(self.context))

    index = ViewPageTemplateFile("rsslink.pt")


class CanonicalURL(ViewletBase):
    """Defines a canonical link relation viewlet to be displayed across the
    site. A canonical page is the preferred version of a set of pages with
    highly similar content. For more information, see:
    https://tools.ietf.org/html/rfc6596
    https://support.google.com/webmasters/answer/139394?hl=en
    """

    @view.memoize
    def render(self):
        context_state = getMultiAdapter(
            (self.context, self.request), name="plone_context_state"
        )
        canonical_url = context_state.canonical_object_url()
        return '    <link rel="canonical" href="%s" />' % canonical_url



from .interfaces import IPortalState
from Acquisition import aq_inner
from plone.app.layout.navigation.root import getNavigationRoot
from plone.app.layout.navigation.root import getNavigationRootObject
from plone.i18n.interfaces import ILanguageSchema
from plone.memoize.view import memoize
from plone.memoize.view import memoize_contextless
from plone.registry.interfaces import IRegistry
from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import IPloneSiteRoot
from plone.base.interfaces import ISearchSchema
from plone.base.interfaces import ISiteSchema
from Products.Five.browser import BrowserView
from zope.component import getUtility
from zope.component import providedBy
from zope.component.hooks import getSite
from zope.interface import implementer


RIGHT_TO_LEFT = ["ar", "fa", "he", "ps"]


@implementer(IPortalState)
class PortalState(BrowserView):
    """Information about the state of the portal"""

    @memoize_contextless
    def portal(self):
        closest_site = getSite()
        if closest_site is not None:
            for potential_portal in closest_site.aq_chain:
                if ISiteRoot in providedBy(potential_portal):
                    return potential_portal
        return None

    @memoize_contextless
    def portal_title(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSchema, prefix="plone", check=False)
        return settings.site_title

    @memoize_contextless
    def portal_url(self):
        return self.portal().absolute_url()

    @memoize
    def navigation_root(self):
        context = aq_inner(self.context)
        portal = self.portal()
        return getNavigationRootObject(context, portal)

    @memoize
    def navigation_root_title(self):
        navigation_root = self.navigation_root()
        if IPloneSiteRoot.providedBy(navigation_root):
            return self.portal_title()

        title = self.navigation_root().Title
        if callable(title):
            return title()
        else:
            return title

    @memoize
    def navigation_root_path(self):
        return getNavigationRoot(aq_inner(self.context))

    @memoize
    def navigation_root_url(self):
        rootPath = self.navigation_root_path()
        return self.request.physicalPathToURL(rootPath)

    @memoize_contextless
    def default_language(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ILanguageSchema, prefix="plone")
        return settings.default_language

    def language(self):
        return (
            self.request.get("LANGUAGE", None)
            or aq_inner(self.context).Language()
            or self.default_language()
        )

    def locale(self):
        return self.request.locale

    def is_rtl(self):
        language = self.language()
        if not language:
            return False
        if language[:2] in RIGHT_TO_LEFT:
            return True
        return False

    @memoize_contextless
    def member(self):
        context = aq_inner(self.context)
        tool = getToolByName(context, "portal_membership")
        return tool.getAuthenticatedMember()

    @memoize_contextless
    def anonymous(self):
        context = aq_inner(self.context)
        tool = getToolByName(context, "portal_membership")
        return bool(tool.isAnonymousUser())

    @memoize_contextless
    def friendly_types(self):
        context = aq_inner(self.context)
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISearchSchema, prefix="plone")
        not_searched = settings.types_not_searched

        types = getToolByName(context, "portal_types").listContentTypes()
        return [t for t in types if t not in not_searched]



from plone.app.layout.viewlets.common import PersonalBarViewlet
from plone.app.viewletmanager.manager import OrderedViewletManager
from plone.memoize.instance import memoize
from plone.registry.interfaces import IRegistry
from plone.base.interfaces.controlpanel import ISiteSchema
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import getMultiAdapter
from zope.component import getUtility


class ToolbarViewletManager(OrderedViewletManager):
    custom_template = ViewPageTemplateFile("toolbar.pt")

    @property
    @memoize
    def _settings(self):
        registry = getUtility(IRegistry)
        return registry.forInterface(ISiteSchema, prefix="plone", check=False)

    def base_render(self):
        return super().render()

    def render(self):
        return self.custom_template()

    @property
    @memoize
    def context_state(self):
        return getMultiAdapter((self.context, self.request), name="plone_context_state")

    @property
    @memoize
    def portal_state(self):
        return getMultiAdapter((self.context, self.request), name="plone_portal_state")

    def toolbar_position(self):
        return self._settings.toolbar_position

    def get_personal_bar(self):
        viewlet = PersonalBarViewlet(self.context, self.request, self.__parent__, self)
        viewlet.update()
        return viewlet

    def get_toolbar_logo(self):
        portal_url = self.portal_state.portal_url()
        try:
            logo = self._settings.toolbar_logo
        except AttributeError:
            logo = "/++plone++static/plone-toolbarlogo.svg"
        if not logo:
            logo = "/++plone++static/plone-toolbarlogo.svg"
        return portal_url + logo

    def show_switcher(self):
        return False



from plone.app.layout.testing import FUNCTIONAL_TESTING
from plone.testing import layered

import doctest
import unittest


optionflags = doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE
normal_testfiles = [
    "history.txt",
]


def test_suite():
    suite = unittest.TestSuite()
    suite.addTests(
        [
            layered(
                doctest.DocFileSuite(
                    test,
                    optionflags=optionflags,
                ),
                layer=FUNCTIONAL_TESTING,
            )
            for test in normal_testfiles
        ]
    )
    return suite



from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.app.layout.viewlets.common import ContentViewsViewlet
from plone.app.layout.viewlets.common import GlobalSectionsViewlet
from plone.app.layout.viewlets.common import LogoViewlet
from plone.app.layout.viewlets.common import TitleViewlet
from plone.app.layout.viewlets.common import ViewletBase
from plone.app.layout.viewlets.tests.base import ViewletsFunctionalTestCase
from plone.app.layout.viewlets.tests.base import ViewletsTestCase
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.protect import authenticator as auth
from plone.registry.interfaces import IRegistry
from plone.testing.zope import Browser
from plone.base.interfaces import INonStructuralFolder
from plone.base.interfaces import ISiteSchema
from zope.component import getUtility
from zope.component.hooks import setSite
from zope.interface import alsoProvides
from zope.interface import directlyProvides
from zope.interface import noLongerProvides

import json
import transaction


# Red pixel with filename pixel.png
SITE_LOGO_BASE64 = (
    b"filenameb64:cGl4ZWwucG5n;datab64:iVBORw0KGgoAAAANSUhEUgA"
    b"AAAEAAAABCAIAAACQd1PeAAAADElEQVQI12P4z8AAAAMBAQAY3Y2wAAA"
    b"AAElFTkSuQmCC"
)
# Text for testing that the title is escaped.
ESCAPED = "&lt;script&gt;"
HACKED = '<script>alert("hacked")</script>'


class TestViewletBase(ViewletsTestCase):
    """Test the base class for the viewlets."""

    def test_update(self):
        request = self.layer["request"]
        setRoles(self.portal, TEST_USER_ID, ["Manager", "Member"])
        self.portal.invokeFactory("Folder", "f1")
        context = getattr(self.portal, "f1")
        alsoProvides(context, INavigationRoot)
        viewlet = ViewletBase(context, request, None, None)
        viewlet.update()
        self.assertEqual(viewlet.site_url, "http://nohost/plone")
        self.assertEqual(viewlet.navigation_root_url, "http://nohost/plone/f1")


class TestContentViewsViewlet(ViewletsTestCase):
    """Test the content views viewlet."""

    def setUp(self):
        super().setUp()
        self.folder.invokeFactory("Document", "test", title="Test default page")
        self.folder.title = "Folder"

    def _invalidateRequestMemoizations(self):
        try:
            del self.app.REQUEST.__annotations__
        except AttributeError:
            pass

    def test_set1_on_portal_root(self):
        self._invalidateRequestMemoizations()
        setRoles(self.portal, TEST_USER_ID, ["Manager", "Member"])
        self.app.REQUEST["ACTUAL_URL"] = self.portal.absolute_url()
        view = ContentViewsViewlet(self.portal, self.app.REQUEST, None)
        view.update()
        self.assertEqual(view.tabSet1[0]["id"], "folderContents")

    def test_set1_NonStructuralFolder(self):
        self._invalidateRequestMemoizations()
        setRoles(self.portal, TEST_USER_ID, ["Manager", "Member"])
        self.app.REQUEST["ACTUAL_URL"] = self.folder.absolute_url()
        directlyProvides(self.folder, INonStructuralFolder)
        view = ContentViewsViewlet(self.folder, self.app.REQUEST, None)
        view.update()
        noLongerProvides(self.folder, INonStructuralFolder)
        self.assertEqual(
            1, len([t for t in view.tabSet1 if t["id"] == "folderContents"])
        )

    def test_set1(self):
        self._invalidateRequestMemoizations()
        setRoles(self.portal, TEST_USER_ID, ["Manager", "Member"])
        self.app.REQUEST["ACTUAL_URL"] = "{}/edit?_authenticator={}".format(
            self.folder.test.absolute_url(),
            auth.createToken(),
        )
        view = ContentViewsViewlet(self.folder.test, self.app.REQUEST, None)
        view.update()
        self.assertEqual(
            1, len([t for t in view.tabSet1 if t["id"] == "folderContents"])
        )
        self.assertEqual(["edit"], [t["id"] for t in view.tabSet1 if t["selected"]])


class TestTitleViewsViewlet(ViewletsTestCase):
    """Test the title viewlet."""

    def setUp(self):
        super().setUp()
        self.folder.invokeFactory("Document", "test", title="Test default page")
        self.folder.title = "Folder"

    def _invalidateRequestMemoizations(self):
        try:
            del self.app.REQUEST.__annotations__
        except AttributeError:
            pass

    def test_title_viewlet_on_portal(self):
        """Title viewlet renders navigation root title"""
        self._invalidateRequestMemoizations()
        setRoles(self.portal, TEST_USER_ID, ["Manager", "Member"])
        self.app.REQUEST["ACTUAL_URL"] = self.portal.absolute_url()
        viewlet = TitleViewlet(self.portal, self.app.REQUEST, None)
        viewlet.update()
        self.assertEqual(viewlet.site_title, "Plone site")
        registry = getUtility(IRegistry)
        site_settings = registry.forInterface(ISiteSchema, prefix="plone", check=False)
        site_settings.site_title = "Süper Site"
        self._invalidateRequestMemoizations()
        viewlet.update()
        self.assertEqual(viewlet.site_title, "S\xfcper Site")

    def test_title_viewlet_on_content(self):
        """Title viewlet renders navigation root title"""
        self._invalidateRequestMemoizations()
        setRoles(self.portal, TEST_USER_ID, ["Manager", "Member"])
        self.app.REQUEST["ACTUAL_URL"] = self.folder.test.absolute_url()
        viewlet = TitleViewlet(self.folder.test, self.app.REQUEST, None)
        viewlet.update()
        self.assertEqual(viewlet.site_title, "Test default page &mdash; Plone site")
        registry = getUtility(IRegistry)
        site_settings = registry.forInterface(ISiteSchema, prefix="plone", check=False)
        site_settings.site_title = "Süper Site"
        self._invalidateRequestMemoizations()
        viewlet.update()
        self.assertEqual(viewlet.site_title, "Test default page &mdash; S\xfcper Site")

    def test_title_viewlet_with_navigation_root(self):
        """Title viewlet renders navigation root title"""
        self._invalidateRequestMemoizations()
        setRoles(self.portal, TEST_USER_ID, ["Manager", "Member"])
        self.app.REQUEST["ACTUAL_URL"] = self.folder.test.absolute_url()
        directlyProvides(self.folder, INavigationRoot)
        viewlet = TitleViewlet(self.folder.test, self.app.REQUEST, None)
        viewlet.update()
        self.assertEqual(viewlet.site_title, "Test default page &mdash; Folder")


class TestLogoViewlet(ViewletsTestCase):
    """Test the site logo viewlet."""

    def _set_site(self, context):
        """Set context as a site."""
        # Set the portal's getSiteManager method on context.
        # This is a hackish way to make setSite work without creating a site
        # with five.localsitemanager.
        # ATTENTION: this works only for the purpose of this test.
        context.getSiteManager = self.portal.getSiteManager
        setSite(context)

    def test_logo_viewlet_portal_root_default(self):
        """When no logo is set, and viewlet is opened on a non-navigation root,
        obtain the default one from the portal.
        """
        viewlet = LogoViewlet(self.folder, self.app.REQUEST, None)
        viewlet.update()
        self.assertEqual(viewlet.img_src, f"{self.portal.absolute_url()}/++resource++plone-logo.svg")

    def test_logo_viewlet_portal_root_registry(self):
        """When a logo is set, and viewlet is opened on a non-navigation root,
        obtain the registry logo from the portal.
        """
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSchema, prefix="plone")
        settings.site_logo = SITE_LOGO_BASE64

        viewlet = LogoViewlet(self.folder, self.app.REQUEST, None)
        viewlet.update()
        self.assertTrue(
            viewlet.img_src,
            f"{self.portal.absolute_url()}/@@site-logo/pixel.png",
        )

    def test_logo_viewlet_navigation_root_default(self):
        """When no logo is set, and viewlet is opened on a navigation root,
        obtain the default one from the navigation root.
        """
        self._set_site(self.folder)
        viewlet = LogoViewlet(self.folder, self.app.REQUEST, None)
        viewlet.update()
        self.assertEqual(viewlet.img_src, f"{self.folder.absolute_url()}/++resource++plone-logo.svg")

    def test_viewlet_navigation_root_registry(self):
        """When a logo is set, and viewlet is opened on a navigation root,
        obtain the registry logo from the navigation root.
        """
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSchema, prefix="plone")
        settings.site_logo = SITE_LOGO_BASE64

        # Set fake site after registry setup...
        self._set_site(self.folder)
        viewlet = LogoViewlet(self.folder, self.app.REQUEST, None)
        viewlet.update()
        self.assertTrue(
            viewlet.img_src,
            f"{self.folder.absolute_url()}/@@site-logo/pixel.png",
        )


class TestGlobalSectionsViewlet(ViewletsTestCase):
    """Test the global sections views viewlet."""

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.registry = getUtility(IRegistry)
        self.folder = self.portal["Members"][TEST_USER_ID]
        self.portal.Members.reindexObject()
        self.folder.reindexObject()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def _get_navtree(self, context=None):
        if context is None:
            context = self.portal
        gsv = GlobalSectionsViewlet(context, self.request.clone(), None)
        return gsv.navtree

    def test_globalnav_respects_types_use_view_action_in_listings(self):
        """Test selected tabs with a INavigationroot folder involved"""
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.invokeFactory("Image", "image", title="Söme Image")
        self.portal.invokeFactory("File", "file", title="Some File")
        self.portal.invokeFactory("Document", "doc", title="Some Döcument")
        self.portal.invokeFactory("Collection", "collection", title="Some Collection")
        request = self.layer["request"]
        gsv = GlobalSectionsViewlet(self.portal, request, None)
        gsv.update()
        html = gsv.render()
        self.assertIn('href="http://nohost/plone/collection"', html)
        self.assertNotIn('href="http://nohost/plone/image/view"', html)
        self.assertNotIn('href="http://nohost/plone/file/view"', html)
        self.assertIn('href="http://nohost/plone/doc"', html)

    def test_globalnav_navigation_depth(self):
        """Test selected tabs with a INavigationroot folder involved"""
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        registry = getUtility(IRegistry)
        registry["plone.navigation_depth"] = 3
        self.portal.invokeFactory("Folder", "folder", title="Földer")
        self.portal.invokeFactory("Folder", "folder2", title="Folder 2")
        self.portal.invokeFactory("Folder", "folder3", title="Folder 3")
        folder = self.portal.folder
        folder.invokeFactory("Folder", "subfolder", title="Subfolder")
        folder.invokeFactory("Folder", "subfolder2", title="Sübfolder 2")
        subfolder = folder.subfolder
        subfolder.invokeFactory("Folder", "subsubfolder", title="Sub2folder")

        request = self.layer["request"]
        navtree = self._get_navtree()
        self.assertListEqual(
            sorted(navtree),
            ["/plone", "/plone/Members", "/plone/folder", "/plone/folder/subfolder"],
        )
        self.assertListEqual(
            [x["title"] for x in navtree["/plone"]],
            ["Home", "Members", "Földer", "Folder 2", "Folder 3"],
        )
        self.assertListEqual(
            [x["title"] for x in navtree["/plone/folder"]],
            ["Subfolder", "Sübfolder 2"],
        )
        self.assertListEqual(
            [x["title"] for x in navtree["/plone/folder/subfolder"]],
            ["Sub2folder"],
        )

        gsv = GlobalSectionsViewlet(self.portal, request, None)
        gsv.update()
        self.assertTrue(gsv.render())

    def test_default_settings(self):
        self.assertEqual(self.registry["plone.navigation_depth"], 3)
        navtree = self._get_navtree()
        self.assertListEqual(sorted(navtree), ["/plone", "/plone/Members"])
        self.assertListEqual(
            [item["path"] for item in navtree["/plone"]],
            ["/plone/index_html", "/plone/Members"],
        )

    def test_do_not_generate_tabs(self):
        self.registry["plone.generate_tabs"] = False
        navtree = self._get_navtree()
        self.assertListEqual(sorted(navtree), ["/plone"])
        self.assertListEqual(
            [item["path"] for item in navtree["/plone"]],
            ["/plone/index_html"],
        )

    def test_generate_tabs_non_folderish(self):
        self.registry["plone.nonfolderish_tabs"] = False
        self.portal.invokeFactory(
            "Document",
            "test-doc",
            title="A simple document (àèìòù)",
        )
        navtree = self._get_navtree()
        self.assertListEqual(sorted(navtree), ["/plone", "/plone/Members"])
        self.assertListEqual(
            [item["path"] for item in navtree["/plone"]],
            ["/plone/index_html", "/plone/Members"],
        )

    def test_generate_tabs_sorted(self):
        self.portal.invokeFactory(
            "Document",
            "test-doc-2",
            title="Document 2",
        )
        self.portal.invokeFactory(
            "Document",
            "test-doc-1",
            title="Document 1",
        )
        navtree = self._get_navtree()
        # default sorting by position in parent
        self.assertListEqual(sorted(navtree), ["/plone", "/plone/Members"])
        self.assertListEqual(
            [item["path"] for item in navtree["/plone"]],
            [
                "/plone/index_html",
                "/plone/Members",
                "/plone/test-doc-2",
                "/plone/test-doc-1",
            ],
        )

        # check we can sort by title
        self.registry["plone.sort_tabs_on"] = "sortable_title"
        navtree = self._get_navtree()
        self.assertListEqual(sorted(navtree), ["/plone", "/plone/Members"])
        self.assertListEqual(
            [item["path"] for item in navtree["/plone"]],
            [
                "/plone/index_html",
                "/plone/Members",
                "/plone/test-doc-1",
                "/plone/test-doc-2",
            ],
        )

        # check we can reverse sorting
        self.registry["plone.sort_tabs_reversed"] = True
        navtree = self._get_navtree()
        self.assertListEqual(sorted(navtree), ["/plone", "/plone/Members"])
        self.assertListEqual(
            [item["path"] for item in navtree["/plone"]],
            [
                "/plone/index_html",
                "/plone/test-doc-2",
                "/plone/test-doc-1",
                "/plone/Members",
            ],
        )

    def test_generate_tabs_displayed_types(self):
        self.registry["plone.displayed_types"] = (
            "Image",
            "File",
            "Link",
            "News Item",
            "Document",
            "Event",
        )
        navtree = self._get_navtree()
        self.assertListEqual(
            [item["path"] for item in navtree["/plone"]],
            [
                "/plone/index_html",
            ],
        )

    def test_generate_tabs_filter_on_state(self):
        self.registry["plone.filter_on_workflow"] = True
        navtree = self._get_navtree()
        self.assertListEqual(
            [item["path"] for item in navtree["/plone"]],
            [
                "/plone/index_html",
            ],
        )
        self.registry["plone.workflow_states_to_show"] = ("private",)
        navtree = self._get_navtree()
        self.assertListEqual(sorted(navtree), ["/plone", "/plone/Members"])
        self.assertListEqual(
            [item["path"] for item in navtree["/plone"]],
            ["/plone/index_html", "/plone/Members"],
        )

        # Let's check this works also with deep navigation
        self.registry["plone.navigation_depth"] = 2
        navtree = self._get_navtree()
        self.assertListEqual(sorted(navtree), ["/plone", "/plone/Members"])
        self.assertListEqual(
            [item["path"] for item in navtree["/plone"]],
            ["/plone/index_html", "/plone/Members"],
        )
        self.assertListEqual(
            [item["path"] for item in navtree["/plone/Members"]],
            ["/plone/Members/test_user_1_"],
        )

    def test_generate_tabs_exclude_from_nav(self):
        self.portal.invokeFactory(
            "Folder",
            "test-folder",
            title="Test folder",
        )
        self.portal.invokeFactory(
            "Folder",
            "excluded-folder",
            title="Excluded folder",
            exclude_from_nav=True,
        )
        self.portal["excluded-folder"].invokeFactory(
            "Folder",
            "sub-folder",
            title="Sub folder",
        )

        navtree = self._get_navtree()
        self.assertListEqual(
            [item["path"] for item in navtree["/plone"]],
            [
                "/plone/index_html",
                "/plone/Members",
                "/plone/test-folder",
            ],
        )

        # Check also that we we have proper nesting
        self.registry["plone.navigation_depth"] = 2
        navtree = self._get_navtree()
        self.assertListEqual(
            sorted(navtree),
            ["/plone", "/plone/Members", "/plone/excluded-folder"],
        )
        self.assertListEqual(
            [item["path"] for item in navtree["/plone/excluded-folder"]],
            ["/plone/excluded-folder/sub-folder"],
        )

        self.registry["plone.navigation_depth"] = 1
        self.registry["plone.show_excluded_items"] = False
        navtree = self._get_navtree()
        self.assertListEqual(sorted(navtree), ["/plone"])
        self.assertListEqual(
            [item["path"] for item in navtree["/plone"]],
            ["/plone/index_html", "/plone/Members", "/plone/test-folder"],
        )

        # If we increase the navigation depth to 2 the sub folder in the
        # exclude folder it is there but unlinked
        self.registry["plone.navigation_depth"] = 2
        navtree = self._get_navtree()
        self.assertListEqual(
            sorted(navtree),
            ["/plone", "/plone/Members", "/plone/excluded-folder"],
        )
        self.assertListEqual(
            [item["path"] for item in navtree["/plone"]],
            ["/plone/index_html", "/plone/Members", "/plone/test-folder"],
        )
        self.assertListEqual(
            [item["path"] for item in navtree["/plone/excluded-folder"]],
            ["/plone/excluded-folder/sub-folder"],
        )

        self.portal["excluded-folder"]["sub-folder"].exclude_from_nav = True
        self.portal["excluded-folder"]["sub-folder"].reindexObject()
        navtree = self._get_navtree()
        self.assertListEqual(
            sorted(navtree),
            ["/plone", "/plone/Members"],
        )
        self.assertListEqual(
            [item["path"] for item in navtree["/plone"]],
            ["/plone/index_html", "/plone/Members", "/plone/test-folder"],
        )

        # check for 'show_excluded_items' in navtree
        self.registry["plone.show_excluded_items"] = True
        navtree = self._get_navtree()
        # if we're not in (sub)context of an excluded item don't show it
        self.assertListEqual(
            [item["path"] for item in navtree["/plone"]],
            ["/plone/index_html", "/plone/Members", "/plone/test-folder"],
        )
        # but if so, keep the tree
        navtree = self._get_navtree(self.portal["excluded-folder"]["sub-folder"])
        self.assertListEqual(
            [item["path"] for item in navtree["/plone"]],
            [
                "/plone/index_html",
                "/plone/Members",
                "/plone/test-folder",
                "/plone/excluded-folder",
            ],
        )
        self.assertListEqual(
            [item["path"] for item in navtree["/plone/excluded-folder"]],
            ["/plone/excluded-folder/sub-folder"],
        )

    def test_generate_tabs__no_portal_tabs(self):
        """Test for constructing the navigation purely out of a catalog query
        and not using portal tabs at all."""

        class CustomGlobalSectionsViewlet(GlobalSectionsViewlet):
            portal_tabs = []

        self.portal.invokeFactory("Folder", id="folder1", title="Folder 1")

        nav = CustomGlobalSectionsViewlet(self.portal, self.request.clone(), None)
        navtree = nav.navtree

        self.assertEqual(navtree["/plone"][0]["id"], "Members")
        self.assertEqual(navtree["/plone"][1]["id"], "folder1")

    def test_customize_tabs(self):
        """Test for constructing the navigation purely out of a catalog query
        and not using portal tabs at all."""
        self.registry["plone.generate_tabs"] = False

        class CustomGlobalSectionsViewlet(GlobalSectionsViewlet):
            def customize_tab(self, entry, tab):
                entry["title"] = "Homepage"
                return entry

        nav = CustomGlobalSectionsViewlet(self.portal, self.request.clone(), None)
        navtree = nav.navtree

        self.assertEqual(navtree["/plone"][0]["title"], "Homepage")

    def test_customize_entry(self):
        """Test for constructing the navigation purely out of a catalog query
        and not using portal tabs at all."""

        class CustomGlobalSectionsViewlet(GlobalSectionsViewlet):
            portal_tabs = []

            def customize_entry(self, entry, brain):
                entry["title"] = "OKAY"
                return entry

        self.portal.invokeFactory("Folder", id="folder1", title="Folder 1")

        nav = CustomGlobalSectionsViewlet(self.portal, self.request.clone(), None)
        navtree = nav.navtree

        self.assertEqual(navtree["/plone"][0]["title"], "OKAY")
        self.assertEqual(navtree["/plone"][1]["title"], "OKAY")

    def test_customize_query(self):
        """Test for constructing the navigation purely out of a catalog query
        and not using portal tabs at all."""

        class CustomGlobalSectionsViewlet(GlobalSectionsViewlet):
            portal_tabs = []

            def customize_query(self, query):
                query["id"] = "folder1"
                return query

        self.portal.invokeFactory(
            "Folder", id="folder1", title="Folder 1", language="sl"
        )
        self.portal.invokeFactory("Folder", id="folder2", title="Folder 2")

        nav = CustomGlobalSectionsViewlet(self.portal, self.request.clone(), None)
        navtree = nav.navtree

        self.assertEqual(len(navtree["/plone"]), 1)
        self.assertEqual(navtree["/plone"][0]["title"], "Folder 1")

    def test_types_using_view(self):
        """Test for constructing the navigation purely out of a catalog query
        and not using portal tabs at all."""

        class CustomGlobalSectionsViewlet(GlobalSectionsViewlet):
            portal_tabs = []
            types_using_view = ["Folder"]

        self.portal.invokeFactory("Folder", id="folder1", title="Folder 1")

        nav = CustomGlobalSectionsViewlet(self.portal, self.request.clone(), None)
        navtree = nav.navtree

        self.assertListEqual(nav.types_using_view, ["Folder"])
        self.assertEqual(
            navtree["/plone"][0]["url"], "http://nohost/plone/Members/view"
        )
        self.assertEqual(
            navtree["/plone"][1]["url"], "http://nohost/plone/folder1/view"
        )

    def test_escaping_twice_does_not_double_escape_items(self):
        """Test for https://github.com/plone/plone.app.layout/issues/280."""

        self.portal.invokeFactory(
            "Document",
            "test-doc-1",
            title="Document 1 & 2",
        )

        request = self.layer["request"]
        gsv = GlobalSectionsViewlet(self.portal, request, None)
        gsv.update()
        html = gsv.render()
        self.assertIn("Document 1 &amp; 2", html)

        # Render again, as this is what happens when an error view is rendered
        # Before the fix this test will fail, as it produces a title with
        # 'Document 1 &amp;amp; 2'.
        html = gsv.render()
        self.assertNotIn("Document 1 &amp;amp; 2", html)
        self.assertIn("Document 1 &amp; 2", html)


class TestTitleEscape(ViewletsFunctionalTestCase):
    """Test that the title in the global sections viewlet is escaped.

    From PloneHotfix20200121, see
    https://plone.org/security/hotfix/20200121/xss-in-the-title-field-on-plone-5-0-and-higher
    """

    def get_browser(self):
        browser = Browser(self.layer["app"])
        browser.handleErrors = False
        browser.addHeader(
            "Authorization",
            f"Basic {SITE_OWNER_NAME}:{SITE_OWNER_PASSWORD}",
        )
        return browser

    def assert_not_hacked(self, browser):
        body = browser.contents
        hacked = HACKED
        if not browser.isHtml:
            # Assume json.
            hacked = json.dumps(HACKED)

        # This gives a too verbose error message, showing the entire body:
        # self.assertNotIn(HACKED, body)
        # So we roll our own less verbose version.
        if hacked in body:
            index = body.index(hacked)
            start = max(0, index - 50)
            end = min(index + len(hacked) + 50, len(body))
            assert False, "Hacked script found in body: ... {} ...".format(
                body[start:end]
            )

        # The escaped version of the HACKED text should be in the response text.
        self.assertIn(ESCAPED, body)

    def test_normal_title(self):
        # Create a folder and page with normal title.
        normal = "'Normal title"
        setRoles(self.portal, TEST_USER_ID, ["Manager", "Member"])
        self.portal.invokeFactory("Folder", id="folder1", title=normal)
        folder1 = self.portal.folder1
        self.assertEqual(folder1.Title(), normal)
        folder1.invokeFactory("Document", id="page1", title=normal)
        page1 = folder1.page1
        self.assertEqual(page1.Title(), normal)
        transaction.commit()

        # Check the output of the normal case for comparison.
        browser = self.get_browser()
        browser.open(folder1.absolute_url())
        self.assertNotIn(ESCAPED, browser.contents)
        browser.open(page1.absolute_url())
        self.assertNotIn(ESCAPED, browser.contents)
        browser.open(folder1.absolute_url() + "/folder_contents")
        self.assertNotIn(ESCAPED, browser.contents)

    def test_xss_from_title(self):
        # Does a script tag as title show up anywhere in the html?
        # It might end up in the main menu.

        # Create a folder and page with a hacked title.
        setRoles(self.portal, TEST_USER_ID, ["Manager", "Member"])
        self.portal.invokeFactory("Folder", id="folder2", title=HACKED)
        folder2 = self.portal.folder2
        self.assertEqual(folder2.Title(), HACKED)
        folder2.invokeFactory("Document", id="page2", title=HACKED)
        page2 = folder2.page2
        self.assertEqual(page2.Title(), HACKED)
        transaction.commit()

        # Check the output of this hacked case.
        browser = self.get_browser()
        browser.open(folder2.absolute_url())
        self.assert_not_hacked(browser)
        browser.open(page2.absolute_url())
        self.assert_not_hacked(browser)
        browser.open(folder2.absolute_url() + "/folder_contents")
        self.assert_not_hacked(browser)



from plone.registry.interfaces import IRegistry
from plone.base.interfaces import ISiteSchema
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import getUtility
from zope.interface import implementer
from zope.viewlet.interfaces import IViewlet


@implementer(IViewlet)
class AnalyticsViewlet(BrowserView):

    render = ViewPageTemplateFile("view.pt")

    def __init__(self, context, request, view, manager):
        super().__init__(context, request)
        self.__parent__ = view
        self.view = view
        self.manager = manager

    @property
    def webstats_js(self):
        registry = getUtility(IRegistry)
        site_settings = registry.forInterface(ISiteSchema, prefix="plone", check=False)
        try:
            return site_settings.webstats_js or ""
        except AttributeError:
            return ""

    def update(self):
        """The viewlet manager _updateViewlets requires this method"""
        pass



from DateTime import DateTime
from plone.app.layout.viewlets.content import ContentRelatedItems
from plone.app.layout.viewlets.content import DocumentBylineViewlet
from plone.app.layout.viewlets.content import HistoryByLineView
from plone.app.layout.viewlets.tests.base import ViewletsTestCase
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_PASSWORD
from plone.locking.interfaces import ILockable
from plone.registry.interfaces import IRegistry
from plone.base.interfaces import ISecuritySchema
from plone.base.interfaces import ISiteSchema
from Products.CMFCore.utils import getToolByName
from z3c.relationfield import RelationValue
from zope.component import getUtility
from zope.interface import Interface
from zope.intid.interfaces import IIntIds


try:
    import pkg_resources

    pkg_resources.get_distribution("plone.app.relationfield")
except pkg_resources.DistributionNotFound:
    HAS_DEXTERITY = False
    pass
else:
    HAS_DEXTERITY = True
    from plone.dexterity.fti import DexterityFTI

    class IMyDexterityItem(Interface):
        """Dexterity test type"""


class TestDocumentBylineViewletView(ViewletsTestCase):
    """
    Test the document by line viewlet
    """

    def setUp(self):
        super().setUp()
        self.folder.invokeFactory("Document", "doc1", title="Document 1")
        self.context = self.folder["doc1"]

        registry = getUtility(IRegistry)
        self.site_settings = registry.forInterface(
            ISiteSchema,
            prefix="plone",
        )
        self.security_settings = registry.forInterface(
            ISecuritySchema,
            prefix="plone",
        )

    def _get_viewlet(self):
        request = self.app.REQUEST
        viewlet = DocumentBylineViewlet(self.context, request, None, None)
        viewlet.update()
        return viewlet

    def test_get_memberinfo(self):
        viewlet = self._get_viewlet()
        self.assertIsNone(viewlet.get_member_info("bogus"))
        self.assertIsInstance(viewlet.get_member_info("test_user_1_"), dict)

    def test_get_url_path(self):
        viewlet = self._get_viewlet()
        self.assertEqual(viewlet.get_url_path("bogus"), "")
        self.assertEqual(viewlet.get_url_path("test_user_1_"), "author/test_user_1_")

        # users with a slash in the userid will have a different URL
        portal_membership = getToolByName(self.portal, "portal_membership")
        portal_membership.addMember("foo/bar", TEST_USER_PASSWORD, ["Member"], "")
        self.assertEqual(viewlet.get_url_path("foo/bar"), "author/?author=foo%2Fbar")

    def test_get_fullname(self):
        viewlet = self._get_viewlet()
        # For non existent user we return the user id
        self.assertEqual(viewlet.get_fullname("bogus"), "bogus")
        # If the fullname is not set we return the user id
        self.assertEqual(viewlet.get_fullname("test_user_1_"), "test_user_1_")

        # otherwise we will return the fullname property
        portal_membership = getToolByName(self.portal, "portal_membership")
        portal_membership.addMember(
            "foo/bar", TEST_USER_PASSWORD, ["Member"], "", properties={"fullname": "Foo Bar"}
        )
        self.assertEqual(viewlet.get_fullname("foo/bar"), "Foo Bar")

    def test_pub_date(self):
        # configure our portal to enable publication date on pages globally on
        # the site
        self.site_settings.display_publication_date_in_byline = True

        logout()
        viewlet = self._get_viewlet()

        # publication date should be None as there is not Effective date set
        # for our document yet
        self.assertEqual(viewlet.pub_date(), None)

        # now set effective date for our document
        effective = DateTime()
        self.context.setEffectiveDate(effective)
        self.assertEqual(viewlet.pub_date(), DateTime(effective.ISO8601()))

        # now switch off publication date globally on the site and see if
        # viewlet returns None for publication date
        self.site_settings.display_publication_date_in_byline = False
        self.assertEqual(viewlet.pub_date(), None)

    def test_anonymous_users_see_byline_if_show_enabled(self):
        self.site_settings.display_publication_date_in_byline = True
        logout()
        viewlet = self._get_viewlet()
        self.assertTrue(viewlet.show())

    def test_anonymous_users_dont_see_byline_if_show_disabled(self):
        self.site_settings.display_publication_date_in_byline = False
        logout()
        viewlet = self._get_viewlet()
        self.assertFalse(viewlet.show())

    def test_logged_users_see_byline_if_show_enabled(self):
        self.site_settings.display_publication_date_in_byline = True
        viewlet = self._get_viewlet()
        self.assertTrue(viewlet.show())

    def test_logged_users_see_byline_if_show_disabled(self):
        self.site_settings.display_publication_date_in_byline = False
        viewlet = self._get_viewlet()
        self.assertTrue(viewlet.show())

    def test_anonymous_users_see_about_if_show_enabled(self):
        self.security_settings.allow_anon_views_about = True
        logout()
        viewlet = self._get_viewlet()
        self.assertTrue(viewlet.show_about())

    def test_anonymous_users_dont_see_about_if_show_disabled(self):
        self.security_settings.allow_anon_views_about = False
        logout()
        viewlet = self._get_viewlet()
        self.assertFalse(viewlet.show_about())

    def test_logged_users_see_about_if_show_enabled(self):
        self.security_settings.allow_anon_views_about = True
        viewlet = self._get_viewlet()
        self.assertTrue(viewlet.show_about())

    def test_logged_users_see_about_if_show_disabled(self):
        self.security_settings.allow_anon_views_about = False
        viewlet = self._get_viewlet()
        self.assertTrue(viewlet.show_about())


class TestHistoryBylineViewletView(ViewletsTestCase):
    """
    Test the document by line viewlet
    """

    def setUp(self):
        super().setUp()
        self.folder.invokeFactory("Document", "doc1", title="Document 1")
        self.context = self.folder["doc1"]

        registry = getUtility(IRegistry)
        self.security_settings = registry.forInterface(
            ISecuritySchema,
            prefix="plone",
        )

    def _get_viewlet(self):
        request = self.app.REQUEST
        viewlet = HistoryByLineView(self.context, request)
        viewlet.update()
        return viewlet

    def test_show_anonymous_not_allowed(self):
        self.security_settings.allow_anon_views_about = False
        logout()
        viewlet = self._get_viewlet()
        self.assertFalse(viewlet.show())

    def test_show_anonymous_allowed(self):
        self.security_settings.allow_anon_views_about = True
        logout()
        viewlet = self._get_viewlet()
        self.assertTrue(viewlet.show())

    def test_show_logged_in_anonymous_not_allowed(self):
        self.security_settings.allow_anon_views_about = False
        viewlet = self._get_viewlet()
        self.assertTrue(viewlet.show())

    def test_show_logged_in_anonymous_allowed(self):
        self.security_settings.allow_anon_views_about = True
        viewlet = self._get_viewlet()
        self.assertTrue(viewlet.show())

    def test_anonymous_locked_icon_not_locked(self):
        logout()
        viewlet = self._get_viewlet()
        self.assertEqual(viewlet.locked_icon(), "")

    def test_anonymous_locked_icon_is_locked(self):
        logout()
        ILockable(self.context).lock()
        viewlet = self._get_viewlet()
        self.assertEqual(viewlet.locked_icon(), "")

    def test_logged_in_locked_icon_not_locked(self):
        viewlet = self._get_viewlet()
        self.assertEqual(viewlet.locked_icon(), "")

    def test_logged_in_locked_icon_is_locked(self):
        viewlet = self._get_viewlet()
        ILockable(self.context).lock()
        lockIconUrl = '<img src="http://nohost/plone/lock_icon.png" alt="" \
title="Locked" height="16" width="16" />'
        self.assertEqual(viewlet.locked_icon(), lockIconUrl)

    def test_pub_date(self):
        # configure our portal to enable publication date on pages globally on
        # the site
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSchema, prefix="plone")

        settings.display_publication_date_in_byline = True

        logout()
        viewlet = self._get_viewlet()

        # publication date should be None as there is not Effective date set
        # for our document yet
        self.assertEqual(viewlet.pub_date(), None)

        # now set effective date for our document
        effective = DateTime()
        self.context.setEffectiveDate(effective)
        self.assertEqual(viewlet.pub_date(), DateTime(effective.ISO8601()))

        # now switch off publication date globally on the site and see if
        # viewlet returns None for publication date
        settings.display_publication_date_in_byline = False
        self.assertEqual(viewlet.pub_date(), None)


class TestRelatedItemsViewlet(ViewletsTestCase):
    def setUp(self):
        super().setUp()
        self.folder.invokeFactory("Document", "doc1", title="Document 1")
        self.folder.invokeFactory("Document", "doc2", title="Document 2")
        self.folder.invokeFactory("Document", "doc3", title="Document 3")
        intids = getUtility(IIntIds)
        self.folder.doc1.relatedItems = [
            RelationValue(intids.getId(self.folder.doc2)),
            RelationValue(intids.getId(self.folder.doc3)),
        ]

    def testRelatedItems(self):
        request = self.app.REQUEST
        viewlet = ContentRelatedItems(self.folder.doc1, request, None, None)
        viewlet.update()
        related = viewlet.related_items()
        self.assertEqual([x.Title for x in related], ["Document 2", "Document 3"])

    def testDeletedRelatedItems(self):
        # Deleted related items should not cause problems.
        self.folder._delObject("doc2")
        request = self.app.REQUEST
        viewlet = ContentRelatedItems(self.folder.doc1, request, None, None)
        viewlet.update()
        related = viewlet.related_items()
        self.assertEqual([x.Title for x in related], ["Document 3"])


class TestDexterityRelatedItemsViewlet(ViewletsTestCase):
    def setUp(self):
        super().setUp()
        """ create some sample content to test with """
        from plone.base.utils import get_installer

        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        fti = DexterityFTI("Dexterity Item with relatedItems behavior")
        self.portal.portal_types._setObject(
            "Dexterity Item with relatedItems behavior", fti
        )
        fti.klass = "plone.dexterity.content.Item"
        test_module = "plone.app.layout.viewlets.tests.test_content"
        fti.schema = test_module + ".IMyDexterityItem"
        fti.behaviors = ("plone.app.relationfield.behavior.IRelatedItems",)
        fti = DexterityFTI("Dexterity Item without relatedItems behavior")
        self.portal.portal_types._setObject(
            "Dexterity Item without relatedItems behavior", fti
        )
        fti.klass = "plone.dexterity.content.Item"
        fti.schema = test_module + ".IMyDexterityItem"
        self.folder.invokeFactory("Document", "doc1", title="Document 1")
        self.folder.invokeFactory("Document", "doc2", title="Document 2")
        self.folder.invokeFactory("Dexterity Item with relatedItems behavior", "dex1")
        self.folder.invokeFactory("Dexterity Item with relatedItems behavior", "dex2")
        self.folder.invokeFactory(
            "Dexterity Item without relatedItems behavior", "dex3"
        )
        qi = get_installer(self.portal)
        qi.install_product("plone.app.intid")
        intids = getUtility(IIntIds)
        self.folder.dex1.relatedItems = [
            RelationValue(intids.getId(self.folder.doc1)),
            RelationValue(intids.getId(self.folder.doc2)),
        ]

    def testDexterityRelatedItems(self):
        request = self.app.REQUEST
        viewlet = ContentRelatedItems(self.folder.dex1, request, None, None)
        viewlet.update()
        related = viewlet.related_items()
        self.assertEqual([x.id for x in related], ["doc1", "doc2"])

        # TODO: we should test with non-published objects and anonymous
        #       users but current workflow has no transition to make an
        #       item private

    def testDexterityEmptyRelatedItems(self):
        request = self.app.REQUEST
        viewlet = ContentRelatedItems(self.folder.dex2, request, None, None)
        viewlet.update()
        related = viewlet.related_items()
        self.assertEqual(len(related), 0)

    def testDexterityWithoutRelatedItemsBehavior(self):
        request = self.app.REQUEST
        viewlet = ContentRelatedItems(self.folder.dex2, request, None, None)
        viewlet.update()
        related = viewlet.related_items()
        self.assertEqual(len(related), 0)

    def testDexterityFolderRelatedItems(self):
        """
        Related items viewlet doesn't include related folder's descendants.
        """
        self.assertTrue(self.folder.contentValues(), "Folder is missing descendants")

        intids = getUtility(IIntIds)
        self.folder.dex1.relatedItems = [RelationValue(intids.getId(self.folder))]

        request = self.app.REQUEST
        viewlet = ContentRelatedItems(self.folder.dex1, request, None, None)
        viewlet.update()
        related = viewlet.related_items()
        self.assertEqual(len(related), 1)

    def testDexterityDeletedRelatedItems(self):
        # Deleted related items should not cause problems.
        self.folder._delObject("doc1")
        request = self.app.REQUEST
        viewlet = ContentRelatedItems(self.folder.dex1, request, None, None)
        viewlet.update()
        related = viewlet.related_items()
        self.assertEqual([x.id for x in related], ["doc2"])



from .interfaces import ITools
from plone.memoize.view import memoize_contextless
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from zope.interface import implementer


@implementer(ITools)
class Tools(BrowserView):
    """Common tools"""

    @memoize_contextless
    def actions(self):
        return getToolByName(self.context, "portal_actions")

    @memoize_contextless
    def catalog(self):
        return getToolByName(self.context, "portal_catalog")

    @memoize_contextless
    def membership(self):
        return getToolByName(self.context, "portal_membership")

    @memoize_contextless
    def properties(self):
        return getToolByName(self.context, "portal_properties")

    @memoize_contextless
    def url(self):
        return getToolByName(self.context, "portal_url")

    @memoize_contextless
    def types(self):
        return getToolByName(self.context, "portal_types")

    @memoize_contextless
    def workflow(self):
        return getToolByName(self.context, "portal_workflow")



from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.app.layout.testing import INTEGRATION_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing.helpers import logout
from plone.portlets.interfaces import IPortletType
from plone.registry.interfaces import IRegistry
from plone.base.interfaces.controlpanel import ILinkSchema
from zope.component import getUtility

import os
import unittest
import zope.interface


class TestLayoutView(unittest.TestCase):
    """Tests the global layout view."""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.view = self.portal.restrictedTraverse("@@plone_layout")
        self.portal.invokeFactory("Folder", "folder1")
        self.portal.invokeFactory("Document", "front-page")
        self.portal.setDefaultPage("front-page")

    def testHavePortlets(self):
        have_portlets = self.view.have_portlets
        self.assertEqual(False, have_portlets("plone.leftcolumn"))
        # We have no portlet on the right in Plone 5.0
        self.assertEqual(False, have_portlets("plone.rightcolumn"))

    def testEnableColumns(self):
        # Make sure we can force a column to appear even if there are no
        # portlets
        self.app = self.layer["app"]
        self.app.REQUEST.set("disable_plone.leftcolumn", 0)
        self.assertEqual(True, self.view.have_portlets("plone.leftcolumn"))

    def testDisableColumns(self):
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.app = self.layer["app"]
        # Now add some portlets to be sure we have columns.  For
        # simplicity we want a portlet that has no add form.  Note
        # that apparently the Calender had no add form until Plone
        # 4.3, but since 4.4 it does, so it is not fit to use here.
        portlet = getUtility(IPortletType, name="portlets.Login")
        mapping_left = self.portal.restrictedTraverse(
            "++contextportlets++plone.leftcolumn"
        )
        mapping_right = self.portal.restrictedTraverse(
            "++contextportlets++plone.rightcolumn"
        )
        for m in mapping_left.keys():
            del mapping_left[m]
        addview_left = mapping_left.restrictedTraverse("+/" + portlet.addview)

        for m in mapping_right.keys():
            del mapping_right[m]
        addview_right = mapping_right.restrictedTraverse("+/" + portlet.addview)

        # This is a NullAddForm - calling it does the work
        addview_left()
        addview_right()

        # Logout, otherwise the login portlet will never show.
        logout()

        # Check that we do not explicitly disable the columns.  This
        # may happen if we change the request in this method or if the
        # addviews return an add-form after all.
        self.assertTrue("disable_plone.leftcolumn" not in self.app.REQUEST)
        self.assertTrue("disable_plone.rightcolumn" not in self.app.REQUEST)

        self.assertEqual(True, self.view.have_portlets("plone.leftcolumn"))
        self.app.REQUEST.set("disable_plone.leftcolumn", 1)
        self.assertEqual(False, self.view.have_portlets("plone.leftcolumn"))

        self.assertEqual(True, self.view.have_portlets("plone.rightcolumn"))
        self.app.REQUEST.set("disable_plone.rightcolumn", 1)
        self.assertEqual(False, self.view.have_portlets("plone.rightcolumn"))

    def testBodyClass(self):
        context = self.portal["front-page"]
        view = context.restrictedTraverse("view")
        layout_view = context.restrictedTraverse("@@plone_layout")
        body_class = layout_view.bodyClass(view, layout_view)
        assert "section-front-page" in body_class

    def testBodyClassTemplate(self):
        # test of first parameter only
        context = self.portal["front-page"]

        view = context.restrictedTraverse("view")
        layout_view = context.restrictedTraverse("@@plone_layout")

        # case 1: name from first parameter, expected a template or view
        from Products.CMFCore.FSPageTemplate import FSPageTemplate

        template = FSPageTemplate(
            "document_view",
            os.path.join(os.path.dirname(__file__), "data", "bodyclass_nametest.pt"),
        )
        body_class = layout_view.bodyClass(template, view)
        self.assertIn("template-document_view", body_class)

        # case 2: even w/o second parameter it has to work
        body_class = layout_view.bodyClass(template, None)
        self.assertIn("template-document_view", body_class)

        # case 3: if theres no template get name from view
        body_class = layout_view.bodyClass(None, view)
        self.assertIn("template-view", body_class)

    def testBodyClassWithNavigationRoot(self):
        # mark a folder "between" self.folder and self.portal with
        # INavigationRoot
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        zope.interface.alsoProvides(self.portal.folder1, INavigationRoot)
        context = self.portal.folder1
        view = context.restrictedTraverse("@@plone_layout")
        template = context.restrictedTraverse("view")
        body_class = view.bodyClass(template, view)
        assert "site-%s" % context.getId() in body_class

    def testBodyClassWithEverySection(self):
        # mark a folder "between" self.folder and self.portal with
        # INavigationRoot
        zope.interface.alsoProvides(self.portal.folder1, INavigationRoot)
        self.portal.folder1.invokeFactory("Folder", "folder2")
        self.portal.folder1.folder2.invokeFactory("Folder", "folder3")
        self.portal.folder1.folder2.folder3.invokeFactory("Document", "page")
        context = self.portal.folder1.folder2.folder3.page
        view = context.restrictedTraverse("@@plone_layout")
        template = context.restrictedTraverse("view")
        body_class = view.bodyClass(template, view)
        assert "section-folder2 site-folder1" in body_class
        assert " subsection-folder3 subsection-folder3-page" in body_class

    def testBodyClassWithEverySectionTurnedOff(self):
        registry = getUtility(IRegistry)
        registry["plone.app.layout.globals.bodyClass.depth"] = 0
        zope.interface.alsoProvides(self.portal.folder1, INavigationRoot)
        self.portal.folder1.invokeFactory("Folder", "folder2")
        self.portal.folder1.folder2.invokeFactory("Folder", "folder3")
        self.portal.folder1.folder2.folder3.invokeFactory("Document", "page")
        context = self.portal.folder1.folder2.folder3.page
        view = context.restrictedTraverse("@@plone_layout")
        template = context.restrictedTraverse("view")
        body_class = view.bodyClass(template, view)
        assert "subsection-folder2 subsection-folder2-folder3" not in body_class
        assert " subsection-folder2-folder3-page" not in body_class

    def testBodyClassWithMarkSpecialLinksOnOff(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ILinkSchema, prefix="plone", check=False)
        context = self.portal["front-page"]
        template = context.restrictedTraverse("view")
        view = context.restrictedTraverse("@@plone_layout")

        # Case 1
        settings.mark_special_links = False
        settings.external_links_open_new_window = False
        body_class = view.bodyClass(template, view)
        self.assertTrue("pat-markspeciallinks" not in body_class)

        # Case 2
        settings.mark_special_links = True
        settings.external_links_open_new_window = False
        body_class = view.bodyClass(template, view)
        self.assertTrue("pat-markspeciallinks" in body_class)

        # Case 3
        settings.mark_special_links = False
        settings.external_links_open_new_window = True
        body_class = view.bodyClass(template, view)
        self.assertTrue("pat-markspeciallinks" in body_class)

        # Case 4
        settings.mark_special_links = True
        settings.external_links_open_new_window = True
        body_class = view.bodyClass(template, view)
        self.assertTrue("pat-markspeciallinks" in body_class)

    def testBodyClassColMarker(self):
        context = self.portal["front-page"]
        view = context.restrictedTraverse("view")
        layout_view = context.restrictedTraverse("@@plone_layout")
        body_class = layout_view.bodyClass(view, layout_view)
        assert "col-content" in body_class



from Acquisition import aq_inner
from Acquisition import aq_parent
from plone.app.layout.icons.interfaces import IContentIcon
from plone.memoize import view
from plone.memoize.instance import memoize
from Products.CMFCore.Expression import createExprContext
from Products.CMFCore.utils import getToolByName
from zope.component import getMultiAdapter
from zope.i18n import translate
from zope.interface import implementer


class BaseIcon:
    """Helper base class for html rendering"""

    __allow_access_to_unprotected_subobjects__ = True

    def __call__(self):
        return self.html_tag()

    @memoize
    def html_tag(self):

        if not self.url:
            return None

        tag = '<img width="{}" height="{}" src="{}"'.format(
            self.width,
            self.height,
            self.url,
        )
        if self.title:
            tag += ' title="%s"' % self.title
        if self.description:
            tag += ' alt="%s"' % self.description
        tag += " />"
        return tag


@implementer(IContentIcon)
class CatalogBrainContentIcon(BaseIcon):
    def __init__(self, context, request, brain):
        self.context = context
        self.request = request
        self.brain = brain

    width = 16
    height = 16
    title = None

    @property
    def url(self):
        path = self.brain.getIcon
        if not path:
            return path

        portal_state_view = getMultiAdapter(
            (self.context, self.request), name="plone_portal_state"
        )
        portal_url = portal_state_view.portal_url()
        return f"{portal_url}/{path}"

    @property
    def description(self):
        context = aq_inner(self.context)
        tt = getToolByName(context, "portal_types")
        fti = tt.get(self.brain["portal_type"])
        if fti is not None:
            res = "{} {}".format(
                translate(fti.Title(), context=self.request),
                self._mimetype(),
            )
            return res.strip()
        else:
            return self.brain["portal_type"]

    def _mimetype(self):
        extensions_mimetype = self.extensions_mimetype()
        id = self.brain.getId
        mimetype = ""
        extlength = 0
        for extension in extensions_mimetype.keys():
            if id.endswith(extension):
                # keep the longest extension
                if len(extension) > extlength:
                    mimetype = extensions_mimetype[extension]
                    extlength = len(extension)

        return mimetype

    @view.memoize_contextless
    def extensions_mimetype(self):
        """Return a dict {'.pdf': 'PDF Document', '.ods': '"""
        mtr = getToolByName(self.context, "mimetypes_registry")
        mimetypes = mtr.mimetypes()
        extensions = {}

        for mimetype in mimetypes:
            for extension in mimetype.extensions:
                extensions[extension] = mimetype.name()

        return extensions


@implementer(IContentIcon)
class CMFContentIcon(BaseIcon):
    def __init__(self, context, request, obj):
        self.context = context
        self.request = request
        self.obj = obj

    width = 16
    height = 16
    title = None

    @property
    def url(self):
        path = self.obj.getIcon(1)
        if not path:
            return path

        portal_url = getToolByName(self.context, "portal_url")()
        return f"{portal_url}/{path}"

    @property
    def description(self):
        context = aq_inner(self.context)
        tt = getToolByName(context, "portal_types")
        fti = tt.get(self.obj.portal_type)
        if fti is not None:
            return fti.Title()
        else:
            return self.obj.portal_type


@implementer(IContentIcon)
class FTIContentIcon(BaseIcon):
    def __init__(self, context, request, obj):
        self.context = context
        self.request = request
        self.obj = obj

    width = 16
    height = 16
    title = None

    @property
    def url(self):
        context = self.context
        portal_url = getToolByName(context, "portal_url")
        portal = portal_url.getPortalObject()

        ec = createExprContext(aq_parent(context), portal, context)
        icon = self.obj.getIconExprObject()
        path = ""
        if icon:
            path = icon(ec)
        return path

    @property
    def description(self):
        return self.obj.Title()


@implementer(IContentIcon)
class PloneSiteContentIcon(BaseIcon):
    def __init__(self, context, request, obj):
        self.context = context
        self.request = request
        self.obj = obj

    width = 16
    height = 16
    title = None

    @property
    def url(self):
        portal_url = getToolByName(self.context, "portal_url")()
        portal_state = getMultiAdapter(
            (self.context, self.request), name="plone_portal_state"
        )
        if portal_state.is_rtl():
            return "%s/rtl-site_icon.png" % portal_url
        else:
            return "%s/site_icon.png" % portal_url

    @property
    def description(self):
        return self.obj.Title()


@implementer(IContentIcon)
class DefaultContentIcon(BaseIcon):
    def __init__(self, context, request, obj):
        self.context = context
        self.request = request
        self.obj = obj

    width = 16
    height = 16
    title = None

    @property
    def url(self):
        portal_url = getToolByName(self.context, "portal_url")()
        if self.obj is None:
            return None
        return "%s/error_icon.png" % portal_url

    @property
    def description(self):
        if self.obj is None:
            return None
        return self.obj.Title()



from plone.app.layout.viewlets.common import ViewletBase
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.statusmessages.interfaces import IStatusMessage
from zope.i18nmessageid import MessageFactory


_ = MessageFactory("plone")

MTYPES_DISPLAY = {
    "info": {
        "msg": _("statusmessage_mtype_info", default="Info:"),
        "icon": "plone-statusmessage-info",
        "cssclass": "statusmessage statusmessage-info alert alert-info",
    },
    "warning": {
        "msg": _("statusmessage_mtype_warning", default="Warning:"),
        "icon": "plone-statusmessage-warning",
        "cssclass": "statusmessage statusmessage-warning alert alert-warning",
    },
    "error": {
        "msg": _("statusmessage_mtype_error", default="Error:"),
        "icon": "plone-statusmessage-error",
        "cssclass": "statusmessage statusmessage-error alert alert-danger",
    },
    "danger": {
        "msg": _("statusmessage_mtype_danger", default="Danger:"),
        "icon": "plone-statusmessage-danger",
        "cssclass": "statusmessage statusmessage-danger alert alert-danger",
    },
}


class GlobalStatusMessage(ViewletBase):
    """Display messages to the current user"""

    index = ViewPageTemplateFile("globalstatusmessage.pt")

    def update(self):
        super().update()
        self.status = IStatusMessage(self.request)
        self.messages = self.status.show()

    def display_info_for_mtype(self, mtype):
        """get info for display of an mtype"""
        return MTYPES_DISPLAY.get(mtype, MTYPES_DISPLAY["info"])



from Acquisition import aq_base
from Acquisition import aq_inner
from Acquisition import aq_parent
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from zope.component import getUtility
from zope.component.hooks import getSite


def getNavigationRoot(context, relativeRoot=None):
    """Get the path to the root of the navigation tree.

    If a relativeRoot argument is provided, navigation root is computed from
    portal path and this relativeRoot.

    If no relativeRoot argument is provided, and there is a root value set in
    portal_properties, navigation root path is computed from
    portal path and this root value.

    IMPORTANT !!!
    Previous paragraphs imply relativeRoot is relative to the Plone portal.

    Else, a root must be computed : loop from the context to the portal,
    through parents, looking for an object implementing INavigationRoot.
    Return the path of that root.
    """
    try:
        portal_url = getToolByName(context, "portal_url")
    except AttributeError:
        site = getSite()
        return "/".join(site.getPhysicalPath())

    if relativeRoot is None:
        # fetch from portal_properties
        registry = getUtility(IRegistry)
        relativeRoot = registry.get("plone.root", None)

    # if relativeRoot has a meaningful value,
    if relativeRoot and relativeRoot != "/":
        # use it

        # while taking care of case where
        # relativeRoot is not starting with a '/'
        if relativeRoot[0] != "/":
            relativeRoot = "/" + relativeRoot

        portalPath = portal_url.getPortalPath()
        return portalPath + relativeRoot
    else:
        # compute the root
        portal = portal_url.getPortalObject()
        root = getNavigationRootObject(context, portal)
        return "/".join(root.getPhysicalPath())


def getNavigationRootObject(context, portal):
    obj = context
    while not INavigationRoot.providedBy(obj) and aq_base(obj) is not aq_base(portal):
        parent = aq_parent(aq_inner(obj))
        if parent is None:
            return obj
        obj = parent
    return obj



from plone.formwidget.namedfile.converter import b64decode_file
from plone.registry.interfaces import IRecordModifiedEvent
from plone.registry.recordsproxy import RecordsProxy
from plone.base.interfaces import ISiteSchema
from zope.component import adapter

import mimetypes


@adapter(ISiteSchema, IRecordModifiedEvent)
def updateMimetype(settings: RecordsProxy, event: IRecordModifiedEvent = None):

    if event.record.fieldName != "site_favicon" or not event.record.value:
        return

    filename = b64decode_file(event.newValue)[0]
    mimetype = mimetypes.guess_type(filename)[0] if filename else None
    if mimetype in ("image/x-icon", None):
        # Override incorrect MIME type registered in both PIL and the
        # Products.MimetypesRegistry product.
        mimetype = "image/vnd.microsoft.icon"
    settings.__registry__["plone.site_favicon_mimetype"] = mimetype



from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.app.layout.navigation.root import getNavigationRoot
from plone.app.layout.testing import INTEGRATION_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing.helpers import logout
from plone.i18n.interfaces import ILanguageSchema
from plone.registry.interfaces import IRegistry
from plone.base.interfaces import ISearchSchema
from plone.base.interfaces import ISiteSchema
from zope.component import getUtility
from zope.event import notify
from zope.i18n.locales import locales
from zope.traversing.interfaces import BeforeTraverseEvent

import unittest
import zope.interface


class TestPortalStateView(unittest.TestCase):
    """Ensure that the basic redirector setup is successful."""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.app = self.layer["app"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.folder = self.portal.portal_membership.getHomeFolder(TEST_USER_ID)
        self.view = self.folder.restrictedTraverse("@@plone_portal_state")

    def test_portal(self):
        self.assertEqual(self.view.portal(), self.portal)

    def test_portal_title(self):
        registry = getUtility(IRegistry)
        self.site_settings = registry.forInterface(ISiteSchema, prefix="plone")
        self.site_settings.site_title = "My title"
        self.assertEqual(self.view.portal_title(), "My title")

    def test_portal_url(self):
        self.assertEqual(self.view.portal_url(), self.portal.absolute_url())

    def test_navigation_root(self):
        self.assertEqual(self.view.navigation_root(), self.portal)
        # mark a folder "between" self.folder and self.portal with
        # INavigationRoot
        members = self.portal["Members"]
        zope.interface.alsoProvides(members, INavigationRoot)
        view = members.restrictedTraverse("@@plone_portal_state")
        self.assertEqual(view.navigation_root(), members)

    def test_navigation_root_path(self):
        self.assertEqual(self.view.navigation_root_path(), "/plone")
        self.assertEqual(
            self.view.navigation_root_path(), getNavigationRoot(self.folder)
        )

        # mark a folder "between" self.folder and self.portal with
        # INavigationRoot
        members = self.portal["Members"]
        zope.interface.alsoProvides(members, INavigationRoot)
        view = members.restrictedTraverse("@@plone_portal_state")
        self.assertEqual(view.navigation_root_path(), "/plone/Members")
        self.assertEqual(view.navigation_root_path(), getNavigationRoot(self.folder))

    def test_navigation_root_title(self):
        registry = getUtility(IRegistry)
        self.site_settings = registry.forInterface(ISiteSchema, prefix="plone")
        self.site_settings.site_title = "Portal title"
        self.assertEqual(self.view.navigation_root_title(), "Portal title")
        members = self.portal["Members"]
        # mark a folder "between" self.folder and self.portal with
        # INavigationRoot
        zope.interface.alsoProvides(members, INavigationRoot)
        view = members.restrictedTraverse("@@plone_portal_state")
        self.assertEqual(view.navigation_root_title(), members.Title())

    def test_navigation_root_url(self):
        url = self.app.REQUEST.physicalPathToURL(getNavigationRoot(self.folder))
        self.assertEqual(self.view.navigation_root_url(), "http://nohost/plone")
        self.assertEqual(self.view.navigation_root_url(), url)

        # mark a folder "between" self.folder and self.portal with
        # INavigationRoot

        members = self.portal["Members"]
        zope.interface.alsoProvides(members, INavigationRoot)
        view = members.restrictedTraverse("@@plone_portal_state")
        self.assertEqual(view.navigation_root_url(), "http://nohost/plone/Members")
        url = self.app.REQUEST.physicalPathToURL(getNavigationRoot(members))
        self.assertEqual(view.navigation_root_url(), url)

    def test_default_language(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ILanguageSchema, prefix="plone")
        settings.default_language = "no"
        self.assertEqual(self.view.default_language(), "no")

    def test_language(self):
        self.app.REQUEST.set("LANGUAGE", "no")
        self.assertEqual(self.view.language(), "no")

    def test_locale(self):
        # Set up registry so that no is an accepted language and that the
        # language information from the request is actually being used.
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ILanguageSchema, prefix="plone")
        settings.use_request_negotiation = True
        settings.available_languages.append("no")

        no = locales.getLocale("no", None, None)
        self.app.REQUEST.set("HTTP_ACCEPT_LANGUAGE", "no")

        # Push request through the BeforeTraverseEvent handler again to
        # update language settings
        notify(BeforeTraverseEvent(self.portal, self.portal.REQUEST))
        self.assertEqual(self.view.locale(), no)

    def test_is_not_rtl(self):
        self.app.REQUEST.set("LANGUAGE", "no")
        self.assertEqual(self.view.is_rtl(), False)

    def test_is_rtl(self):
        self.app.REQUEST.set("LANGUAGE", "he")
        self.assertEqual(self.view.is_rtl(), True)
        self.app.REQUEST.set("LANGUAGE", "ar_DZ")
        self.assertEqual(self.view.is_rtl(), True)

    def test_member(self):
        self.assertEqual(
            self.view.member().id,
            self.portal.portal_membership.getAuthenticatedMember().id,
        )

    def test_anonymous(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        # self.portal.invokeFactory('Folder', 'f0')
        self.assertEqual(self.view.anonymous(), False)
        logout()
        del self.app.REQUEST.__annotations__
        self.assertEqual(self.view.anonymous(), True)

    def test_friendly_types(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISearchSchema, prefix="plone")
        settings.types_not_searched = ("Document",)
        self.assertFalse("Document" in self.view.friendly_types())



from plone.app.layout.links.viewlets import RSSViewlet
from plone.app.layout.viewlets.tests.base import ViewletsTestCase
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.base.interfaces import ISiteSyndicationSettings
from plone.registry.interfaces import IRegistry
from zope.component import getUtility


class TestRSSViewletView(ViewletsTestCase):
    def test_RSSViewlet(self):
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.invokeFactory("Folder", "news")
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSyndicationSettings)
        if settings.allowed:
            # make sure syndication is disabled
            settings.allowed = False
        request = self.app.REQUEST
        viewlet = RSSViewlet(self.portal, request, None, None)
        viewlet.update()
        result = viewlet.render()
        self.assertEqual(result.strip(), "")
        settings.allowed = True
        settings.site_rss_items = (self.portal.news.UID(),)
        request = self.app.REQUEST
        viewlet = RSSViewlet(self.portal, request, None, None)
        viewlet.update()
        result = viewlet.render()
        self.assertFalse("<link" not in result)
        self.assertFalse("http://nohost/plone/atom.xml" not in result)
        self.assertFalse("http://nohost/plone/news/atom.xml" not in result)



from setuptools import find_packages
from setuptools import setup


version = "4.0.0b5"

long_description = open("README.rst").read() + "\n" + open("CHANGES.rst").read()

setup(
    name="plone.app.layout",
    version=version,
    description="Layout mechanisms for Plone",
    long_description=long_description,
    classifiers=[
        "Development Status :: 6 - Mature",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 6.0",
        "Framework :: Plone :: Core",
        "Framework :: Zope :: 5",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    keywords="plone layout viewlet",
    author="Plone Foundation",
    author_email="plone-developers@lists.sourceforge.net",
    url="https://pypi.org/project/plone.app.layout",
    license="GPL version 2",
    packages=find_packages(),
    namespace_packages=["plone", "plone.app"],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "Acquisition",
        "DateTime",
        "plone.app.content",
        "plone.app.viewletmanager >=1.2",
        "plone.base",
        "plone.batching >1.0.999",
        "plone.i18n",
        "plone.memoize",
        "plone.portlets",
        "plone.registry",
        "Products.CMFCore",
        "Products.CMFDynamicViewFTI",
        "Products.CMFEditions >=1.2.2",
        "setuptools",
        "zope.component",
        "zope.deferredimport",
        "zope.deprecation",
        "zope.dottedname",
        "zope.i18n",
        "zope.interface",
        "zope.publisher",
        "zope.schema",
        "zope.viewlet",
        "Zope",
    ],
    extras_require=dict(
        test=[
            "plone.app.contenttypes",
            "plone.app.intid",
            "plone.app.relationfield",
            "plone.app.testing",
            "plone.dexterity",
            "plone.locking",
            "plone.testing",
            "z3c.relationfield",
            "zope.annotation",
        ]
    ),
)



from plone.app.layout.viewlets.common import ViewletBase


class HeaderViewlet(ViewletBase):
    """a base viewlet that do not render anything, just ready to set headers"""

    def index(self):
        return ""

    def update(self):
        super().update()
        self.setHeader = self.request.response.setHeader
        for name, value in self.getHeaders():
            self.setHeader(name, value)

    def getHeaders(self):
        return []


class HTTPCachingHeaders(HeaderViewlet):
    """Replace the old global_cache_settings/macros/cacheheaders"""

    def getHeaders(self):
        lang = getattr(self.context, "Language", None)
        if callable(lang):
            lang = lang()
        if not lang:
            lang = self.portal_state.default_language()
        return [
            ("Content-Type", "text/html;charset=utf-8"),
            ("Expires", "Sat, 1 Jan 2000 00:00:00 GMT"),
            ("Content-Language", lang),
        ]



from plone.app.layout.viewlets.common import TitleViewlet
from plone.memoize.view import memoize
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.browser.syndication.adapters import BaseItem
from Products.CMFPlone.browser.syndication.adapters import FolderFeed
from plone.base.interfaces import ISocialMediaSchema
from plone.base.interfaces.syndication import IFeedItem
from Products.CMFPlone.utils import getSiteLogo
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.component.hooks import getSite

import logging

logger = logging.getLogger("plone.app.layout")


class SocialTagsViewlet(TitleViewlet):
    social_image_scale = "great"

    def head_tag_filter(self, value):
        if not isinstance(value, dict):
            return
        return "itemprop" not in value

    def body_tag_filter(self, value):
        if not isinstance(value, dict):
            return
        return "itemprop" in value

    @property
    def tags(self):
        # Do not show items with 'itemprop'.
        return list(filter(self.head_tag_filter, self._get_tags()))

    @property
    def body_tags(self):
        # Show only items without 'itemprop'.
        return list(filter(self.body_tag_filter, self._get_tags()))

    @memoize
    def _get_tags(self):
        site = getSite()
        registry = getUtility(IRegistry)
        settings = registry.forInterface(
            ISocialMediaSchema, prefix="plone", check=False
        )

        if not settings.share_social_data:
            return []

        portal_membership = getToolByName(site, "portal_membership")
        is_anonymous = bool(portal_membership.isAnonymousUser())
        if not is_anonymous:
            return []

        tags = [
            dict(itemprop="name", content=self.page_title),
            dict(name="twitter:card", content="summary"),
            dict(property="og:site_name", content=self.site_title_setting),
            dict(property="og:title", content=self.page_title),
            dict(property="og:type", content="website"),
        ]
        if settings.twitter_username:
            tags.append(
                dict(
                    name="twitter:site",
                    content="@" + settings.twitter_username.lstrip("@"),
                )
            )
        if settings.facebook_app_id:
            tags.append(dict(property="fb:app_id", content=settings.facebook_app_id))
        if settings.facebook_username:
            tags.append(
                dict(
                    property="og:article:publisher",
                    content="https://www.facebook.com/" + settings.facebook_username,
                )
            )

        # reuse syndication since that packages the data
        # the way we'd prefer likely
        feed = FolderFeed(site)
        item = queryMultiAdapter((self.context, feed), IFeedItem, default=None)
        if item is None:
            item = BaseItem(self.context, feed)

        tags.extend(
            [
                dict(itemprop="description", content=item.description),
                dict(itemprop="url", content=item.link),
                dict(property="og:description", content=item.description),
                dict(property="og:url", content=item.link),
            ]
        )

        found_image = False
        if item.has_enclosure and item.file_length > 0:
            if item.file_type.startswith("image"):
                image = None
                scales = self.context.restrictedTraverse('@@images', None)
                if scales:
                    try:
                        image = scales.scale('image', scale=self.social_image_scale)
                    except Exception as e:
                        logger.exception(e)
                if image:
                    tags.extend(
                        [
                            dict(property="og:image", content=image.url),
                            dict(property="og:image:width", content=image.width),
                            dict(property="og:image:height", content=image.height),
                            dict(itemprop="image", content=image.url),
                            dict(property="og:image:type", content=item.file_type),
                        ]
                    )
                    found_image = True
            elif (
                item.file_type.startswith("video")
                or item.file_type == "application/x-shockwave-flash"
            ):
                tags.extend(
                    [
                        dict(property="og:video", content=item.file_url),
                        dict(property="og:video:type", content=item.file_type),
                    ]
                )
            elif item.file_type.startswith("audio"):
                tags.extend(
                    [
                        dict(property="og:audio", content=item.file_url),
                        dict(property="og:audio:type", content=item.file_type),
                    ]
                )

        if not found_image:
            url, mime_type = getSiteLogo(include_type=True)
            tags.extend(
                [
                    dict(property="og:image", content=url),
                    dict(itemprop="image", content=url),
                    dict(property="og:image:type", content=mime_type),
                ]
            )
        return tags



from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_PASSWORD
from Products.CMFPlone.utils import _createObjectByType


class Fixture(PloneSandboxLayer):

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load ZCML
        import plone.app.layout

        self.loadZCML(package=plone.app.layout)

    def setUpPloneSite(self, portal):
        _createObjectByType("Folder", portal, id="Members")
        mtool = portal.portal_membership
        if not mtool.getMemberareaCreationFlag():
            mtool.setMemberareaCreationFlag()
        mtool.createMemberArea(TEST_USER_ID)
        if mtool.getMemberareaCreationFlag():
            mtool.setMemberareaCreationFlag()


FIXTURE = Fixture()
INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE,),
    name="plone.app.layout:Integration",
)
FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE,),
    name="plone.app.layout:Functional",
)
